/*****************************************************************************/
/*                                                                           */
/* AresFit : addpoint.c                                                      */
/*                                                                           */
/* Created:       2014/09/20 (JLe)                                           */
/* Last modified: 2014/10/21 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: Adds a point to matrix A and vector Y                        */
/*                                                                           */
/* Comments:                                                                 */
/*                                                                           */
/*****************************************************************************/

#include "header.h"
#include "locations.h"

#define FUNCTION_NAME "AddPoint:"

/*****************************************************************************/

long AddPoint(long N, long lib, long para, complex **A, complex *Y, 
	      double VHI, double BHI, double BOR, double CR, double TFU, 
	      double VOI, double TMO, double EXP, long inv)
{
  long n, mode, code;
  double x1, x2, x3, x4, x5, x6, val;

  /* Get mode */

  mode = (long)RDB[DATA_MODE];
  code = (long)RDB[DATA_CODE];

  /* Check code */

  if (code == CODE_ARES)
    {
      /***********************************************************************/
      
      /***** Ares polynomial fit *********************************************/

      /* Calculate variables */

      x1 = -VOI;
      x2 = sqrt(TFU) - sqrt(RDB[lib + LIB_NOM_TFU]);
      x3 = TMO - RDB[lib + LIB_NOM_TMO];
      x4 = 0.0;
      x5 = BOR;
      x6 = BHI;

      /* Reset index */
      
      n = 0;
      
      /* Put values matrix */
      
      A[N][n++].re = 1.0;          /* S_base */
      
      if (mode > MODE_HZP)
	{
	  A[N][n++].re = x1*x1;        /* a */
	  A[N][n++].re = x1;           /* b */
	  A[N][n++].re = x2*x2;        /* c */
	  A[N][n++].re = x2;           /* d */
	  A[N][n++].re = x3*x3;        /* e */
	  A[N][n++].re = x3;           /* f */
	  A[N][n++].re = x1*x2;        /* g */
	  A[N][n++].re = x1*x3*x3;     /* h */
	  A[N][n++].re = x1*x3;        /* i */
	  A[N][n++].re = x2*x3;        /* j */
	}

      /* k */
      
      if (mode > MODE_BAS)
	{
	  A[N][n++].re = x5*x5;        /* l */
	  A[N][n++].re = x5;           /* m */
	}
      
      if (mode > MODE_HFP)
	{
	  A[N][n++].re = x6*x6;        /* n */
	  A[N][n++].re = x6;           /* o */
	}
      
      if (mode > MODE_HZP)
	{
	  A[N][n++].re = x5*x5*x3*x3;  /* p */
	  A[N][n++].re = x5*x5*x3;     /* q */
	  A[N][n++].re = x5*x3*x3;     /* v */
	  A[N][n++].re = x5*x3;        /* w */
	  A[N][n++].re = x5*x5*x2;     /* r */
	  A[N][n++].re = x5*x2;        /* s */
	  A[N][n++].re = x5*x5*x1;     /* t */
	  A[N][n++].re = x5*x1;        /* u */
	}
      
      /* Get value */

      val = GetXS(lib, para, VHI, BHI, BOR, CR, TFU, VOI, TMO, EXP);

      /* Put value in vector */
      
      if (val == 0.0)
	{
	  printf("\nZero value: VHI = %1.2f\n", VHI);
	  printf("            BHI = %1.2f\n", BHI);
	  printf("            BOR = %1.2f\n", BOR);
	  printf("            CR  = %1.0f\n", CR);
	  printf("            TFU = %1.2f\n", TFU);
	  printf("            VOI = %1.2f\n", VOI);
	  printf("            TMO = %1.2f\n", TMO);
	  printf("            EXP = %1.2f\n", EXP);
	  printf("\nUnable to proceed.\n\n");
	  exit(0);      
	  
	  Y[N].re = val;
	}
      else
	Y[N].re = val;

      /***********************************************************************/
    }
  else if (code == CODE_APROS)
    {
      /***** APROS polynomial fit ********************************************/
  
      /* Calculate variables (HUOM: tiheys !!!) */

      x1 = sqrt(TFU) - sqrt(RDB[lib + LIB_NOMCON_TFNOM]);
      x2 = TMO - RDB[lib + LIB_NOMCON_DMNOM];
      x3 = BOR - RDB[lib + LIB_NOMCON_CBNOM];

      /* Reset index */
      
      n = 0;
      
      /* Put values matrix */
      
      A[N][n++].re = 1.0;          /* S_base */
      A[N][n++].re = x1;
      A[N][n++].re = x1*x1;
      A[N][n++].re = x2;
      A[N][n++].re = x2*x2;
      A[N][n++].re = x3;
      A[N][n++].re = x3*x3;

      /* Get value */

      val = GetXS(lib, para, VHI, BHI, BOR, CR, TFU, VOI, TMO, EXP);

      /* Invert diffusion coefficient */
      
      if (((para == XS_DIFF1) || (para == XS_DIFF2)) && (inv == YES))
	{
	  /* Check zero */
	  
	  if (val == 0.0)
	    Die(FUNCTION_NAME, "Zero diffusion coefficient");
	  else
	    val = 1.0/val;
	  
	}
      
      /* Put value */

      Y[N].re = val;
	  
      /***********************************************************************/
    }
  else
    Die(FUNCTION_NAME, "Code type %ld not supported", code);

  /* Return new size */

  return N + 1;
}

/*****************************************************************************/

