/*****************************************************************************/
/*                                                                           */
/* AresFit : checkpointer.c                                                  */
/*                                                                           */
/* Created:       2014/08/25 (JLe)                                           */
/* Last modified: 2014/08/25 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: Checks that pointer is within allowed limits                 */
/*                                                                           */
/* Comments: - Adopted from Serpent 2                                        */
/*                                                                           */
/*****************************************************************************/

#include "header.h"
#include "locations.h"

#define FUNCTION_NAME "CheckPointer:"

/*****************************************************************************/

#ifdef DEBUG

void CheckPointer (char *fname, char *comment, long type, long ptr)
{
  long max, min;

  /* Avoid compiler warning */

  min = -1;
  max = -1;

  /* Get minimum and maximum */

  if (abs(type) == DATA_ARRAY)
    {
      min = VALID_PTR;
      max = (long)RDB[DATA_ALLOC_MAIN_SIZE] - 1;
    }
  else
    Die(fname, "Invalid data type %ld. %s", type, comment);

  /* Check pointer */

  if (ptr < 0)
    Die(fname, "Null pointer %s", comment);
  else if ((type < 0) && (ptr != max))
    Die(fname, "Pointer error: value %ld not equal to maximum %s", ptr, 
	comment);
  else if (ptr < min)
    Die(fname, "Pointer error: value %ld below minimum = %ld %s", ptr, min,
	comment);
  else if (ptr > max)
    Die(fname, "Pointer error: value %ld above maximum = %ld %s %ld", ptr, max,
	comment, type);
}

#endif

/*****************************************************************************/
