/*****************************************************************************/
/*                                                                           */
/* AresFit : fit2.c                                                          */
/*                                                                           */
/* Created:       2014/08/25 (JLe)                                           */
/* Last modified: 2014/08/25 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: Fits 2nd order curve to three points                         */ 
/*                                                                           */
/* Comments:                                                                 */
/*                                                                           */
/*****************************************************************************/

#include "header.h"

#define FUNCTION_NAME "Fit2:"

/*****************************************************************************/

double *Fit2(double *x, double *y)
{
  long m;
  double denom;
  static double c[3];

  /* Calculte denominator */
  
  if ((denom = (x[0]*x[0]*(x[1] - x[2]) + x[1]*x[1]*(x[2] - x[0]) + 
		x[2]*x[2]*(x[0] - x[1]))) == 0.0)
    Die(FUNCTION_NAME, "Division by zero");
  
  /* Calculate coefficients */

  c[0] = ((y[0] - y[1])*(x[1] - x[2]) + (y[2] - y[1])*(x[0] - x[1])) / denom;
  c[1] = (y[0] - y[1] - c[0]*(x[0]*x[0] - x[1]*x[1]))/(x[0] - x[1]);
  c[2] = y[0] - c[0]*x[0]*x[0] - c[1]*x[0]; 

  /* Check points */
		  
  for (m = 0; m < 3; m++)
    if (y[m] != 0.0)
      CheckValue(FUNCTION_NAME, "p", fabs(y[m]/Val2(c,x[m]) - 1.0), 0, 1E-5);
  
  /* Return vector */

  return c;
}

/*****************************************************************************/

