/*****************************************************************************/

/***** External libraries ****************************************************/

#define _XOPEN_SOURCE 600
#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include <time.h>

/*
#include <strings.h>
#include <limits.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/timeb.h>
*/

/*****************************************************************************/

/***** Code name and version *************************************************/

#define CODE_NAME     "SXSFit"
#define CODE_VERSION  "1.0.0"
#define CODE_DATE     "February 19, 2015"
#define CODE_AUTHOR   "Jaakko.Leppanen@vtt.fi"

/*****************************************************************************/

/***** Global definitions ****************************************************/

/* Array types */

#define DATA_ARRAY     1 

/* very negative null pointer for data arrays */

#define NULLPTR -1E+6

/* yes and no */

#define YES 1
#define NO  0

/* Sort modes */

#define SORT_MODE_ASCEND   1
#define SORT_MODE_DESCEND  2

/* Small and large values */

#define ZERO        1e-37
#define INFTY       1e+37

/* Data sizes in bytes */

#define KILO        1024.0
#define MEGA     1048576.0
#define GIGA  1073741824.0

/* Memory allocation */

#define MEM_ALLOC    1
#define MEM_REALLOC  2
#define MEM_FREE     3

/* Codes */

#define CODE_ARES   1
#define CODE_APROS  2
#define CODE_PARCS  3

/* Modes */

#define MODE_BAS  0
#define MODE_HZP  1
#define MODE_HFP  2
#define MODE_DEP  3

/* Maximum array sizes */

#define MAX_STR               256

/* Constants */

#define MEV         1.6021765314E-13     /* J per MeV    */
#define BARN        1E-24                /* barn in cm2  */

/* Flags for PARCS */

#define PARCS_FLAG_LADF   1
#define PARCS_FLAG_LXES   2
#define PARCS_FLAG_LDED   4
#define PARCS_FLAG_LJ1F   8
#define PARCS_FLAG_LCHI   16
#define PARCS_FLAG_LCHD   32
#define PARCS_FLAG_LINV   64
#define PARCS_FLAG_LDET   128
#define PARCS_FLAG_LYLD   256
#define PARCS_FLAG_LCDF   512
#define PARCS_FLAG_LGFF   1024
#define PARCS_FLAG_LBET   2048
#define PARCS_FLAG_LAMB   4096
#define PARCS_FLAG_LDEC   8192
#define PARCS_FLAG_LDER   16384

/*****************************************************************************/

/***** Structures and definitions ********************************************/

/* Complex number */

typedef struct {
  double re;
  double im;
} complex;

/*****************************************************************************/

/***** Macros ****************************************************************/

/* These are used for testing */

#define Val2(p,x)(p[0]*x*x + p[1]*x + p[2])

/*****************************************************************************/

/***** Function prototypes ***************************************************/

void AddItem(long, long);

long AddPoint(long, long, long, complex **, complex *, double, double, double,
	      double, double, double, double, double, long);

double AprosPoly(double *, double, double, double);

double ARESPoly(double *, double, double, double, double, double, double);

double AtoF(char *);

long AtoI(char *);

complex c_add (complex, complex); 

complex c_con (complex); 

complex c_div (complex, complex); 

complex c_mul (complex, complex);

double  c_norm(complex);

complex c_sub (complex, complex);

complex c_sqrt(complex);

complex c_exp(complex);

long CompareStr(long, long);

void Die(char *, ...);

double GetXS(long, long, double, double, double, double, double, double, 
	     double, double);

long FirstItem(long);

double *Fit2(double *, double *);

char *GetText(long);

long LastItem(long);

void LUdecomposition(long, complex **, complex *, complex *);

void *Mem(long, ...);

void MoveItemRight(long);

long NewItem(long, long);

FILE *OpenFile(char *, char *);

void ProcessApros();

void ProcessARES();

void ProcessPARCS();

long PutText(char *);

void ReadData();

double ReadF(FILE *);

long ReadI(FILE *);

long ReallocMem(long, long);

void Reconfigure();

void RemoveItem(long);

long SeekListStr(long, long, char *);

void SetDirectPointers(long);

void SortList(long, long, long);

void TestAres(long, long, double, double, double, double, double *);

void TestApros(long, long, double, double, double, double, double *, long, 
	       long, long);

char *TimeStamp();

/*****************************************************************************/

/***** Functions replaced by macros in non-debug mode ************************/

#ifdef DEBUG

void CheckPointer (char *, char *, long, long);

void CheckValue(char *, char *, double, double, double);

long NextItem(long);

long PrevItem(long);

#else

#define CheckPointer(a, b, c, d)

#define CheckValue(a, c, d, e, f)

#define NextItem(a)((long)RDB[a + LIST_PTR_NEXT])

#define PrevItem(a)((long)RDB[a + LIST_PTR_PREV])

#endif

/* Other macros */

#define ListCommon(a)((long)RDB[a + LIST_PTR_COMMON])

#define ListSize(a)((long)RDB[ListCommon(a) + LIST_COMMON_N_ITEMS])

/*****************************************************************************/

/***** Global variables ******************************************************/

/* Variables */

FILE *err;

/* Arrays */

double *WDB;
const double *RDB;
char *ASCII;

/*****************************************************************************/
