/*****************************************************************************/

/***** Main data array (direct pointers) *************************************/

/* Data sizes */

#define DATA_ASCII_DATA_SIZE              0
#define DATA_ALLOC_MAIN_SIZE              1
#define DATA_REAL_MAIN_SIZE               3
#define DATA_TOTAL_BYTES                  4

/* Pointers */

#define DATA_PTR_LIB0                    20
#define DATA_PTR_XS0                     21

/* Parameters */

#define DATA_CODE                        29
#define DATA_MODE                        30

/* Temporary pointer */

#define DATA_TMP_PTR                     50
#define DATA_TMP_PTR1                    51
#define DATA_TMP_PTR2                    52

#define DATA_LAST_VALUE                  80

#define VALID_PTR      (DATA_LAST_VALUE - 1)

/*****************************************************************************/

/***** List data *************************************************************/

/* This is data stored for every item */

#define LIST_DATA_SIZE   4

#define LIST_PTR_NEXT    0
#define LIST_PTR_PREV    1
#define LIST_PTR_COMMON  2
#define LIST_PTR_DIRECT  3

/* This is common data in a separate structure */

#define LIST_COMMON_DATA_SIZE  5

#define LIST_COMMON_ITEM_SIZE      0
#define LIST_COMMON_PTR_ROOT       1
#define LIST_COMMON_N_ITEMS        2
#define LIST_COMMON_PTR_FIRST      3
#define LIST_COMMON_PTR_LAST       4

/*****************************************************************************/

/***** Library ***************************************************************/

#define LIB_BLOCK_SIZE                (LIST_DATA_SIZE + 51)

#define LIB_PTR_NAME                  (LIST_DATA_SIZE + 0)
#define LIB_PTR_HIS                   (LIST_DATA_SIZE + 1)
#define LIB_NOM_VOI                   (LIST_DATA_SIZE + 2)
#define LIB_NOM_TFU                   (LIST_DATA_SIZE + 3)
#define LIB_NOM_TMO                   (LIST_DATA_SIZE + 4)
#define LIB_PARA_VHI                  (LIST_DATA_SIZE + 5)
#define LIB_PARA_BHI                  (LIST_DATA_SIZE + 6)
#define LIB_PARA_BOR                  (LIST_DATA_SIZE + 7)
#define LIB_PARA_CR                   (LIST_DATA_SIZE + 8)
#define LIB_PARA_TFU                  (LIST_DATA_SIZE + 9)
#define LIB_PARA_VOI                  (LIST_DATA_SIZE + 10)
#define LIB_PARA_TMO                  (LIST_DATA_SIZE + 11)
#define LIB_PARA_EXP                  (LIST_DATA_SIZE + 12)
#define LIB_PARA_RHO                  (LIST_DATA_SIZE + 13)
#define LIB_N_EXP                     (LIST_DATA_SIZE + 14)
#define LIB_PTR_TIT                   (LIST_DATA_SIZE + 15)
#define LIB_MATER_MNUM                (LIST_DATA_SIZE + 16)
#define LIB_MATER_UMASS               (LIST_DATA_SIZE + 17)
#define LIB_NOMCON_PNOM               (LIST_DATA_SIZE + 18)
#define LIB_NOMCON_KXE                (LIST_DATA_SIZE + 19)
#define LIB_NOMCON_KSM                (LIST_DATA_SIZE + 20)
#define LIB_NOMCON_TFNOM              (LIST_DATA_SIZE + 21)
#define LIB_NOMCON_TMNOM              (LIST_DATA_SIZE + 22)
#define LIB_NOMCON_TINOM              (LIST_DATA_SIZE + 23)
#define LIB_NOMCON_CBNOM              (LIST_DATA_SIZE + 24)
#define LIB_NOMCON_DMNOM              (LIST_DATA_SIZE + 25)
#define LIB_NOMCON_DINOM              (LIST_DATA_SIZE + 26)
#define LIB_NOMCON_ALFA               (LIST_DATA_SIZE + 27)
#define LIB_NOMCON_MODC               (LIST_DATA_SIZE + 28)
#define LIB_NOMCON_NHET               (LIST_DATA_SIZE + 29)
#define LIB_FITCO_NXE                 (LIST_DATA_SIZE + 30)
#define LIB_FITCO_NSM                 (LIST_DATA_SIZE + 31)
#define LIB_FITCO_PTR_DOPPLER         (LIST_DATA_SIZE + 32)
#define LIB_FITCO_PTR_BORON           (LIST_DATA_SIZE + 33)
#define LIB_FITCO_PTR_MOD_DENS_TEMP   (LIST_DATA_SIZE + 34)
#define LIB_FITCO_PTR_MOD_DENS        (LIST_DATA_SIZE + 35)
#define LIB_FITCO_PTR_MOD_TEMP        (LIST_DATA_SIZE + 36)
#define LIB_PTR_GCU                   (LIST_DATA_SIZE + 37)
#define LIB_PTR_FILE                  (LIST_DATA_SIZE + 38)
#define LIB_PARCS_FLAGS               (LIST_DATA_SIZE + 39)
#define LIB_NEG                       (LIST_DATA_SIZE + 40)
#define LIB_NDG                       (LIST_DATA_SIZE + 41)
#define LIB_NHG                       (LIST_DATA_SIZE + 42)
#define LIB_NSDF                      (LIST_DATA_SIZE + 43)
#define LIB_NCDF                      (LIST_DATA_SIZE + 44)
#define LIB_INPUT_NSDF                (LIST_DATA_SIZE + 45)
#define LIB_INPUT_NCDF                (LIST_DATA_SIZE + 46)
#define LIB_PARCS_PTR_ID              (LIST_DATA_SIZE + 47)
#define LIB_PARCS_PITCH               (LIST_DATA_SIZE + 48)
#define LIB_PARCS_IHMD                (LIST_DATA_SIZE + 49)
#define LIB_PARCS_DSAT                (LIST_DATA_SIZE + 50)

/*****************************************************************************/

/***** History cases *********************************************************/

#define HIS_BLOCK_SIZE                (LIST_DATA_SIZE + 4)

#define HIS_PTR_FILE                  (LIST_DATA_SIZE + 0)
#define HIS_PTR_XS                    (LIST_DATA_SIZE + 1)
#define HIS_PTR_GCU                   (LIST_DATA_SIZE + 2)
#define HIS_PTR_LIB_NAME              (LIST_DATA_SIZE + 3)

/*****************************************************************************/

/***** Cross section data ****************************************************/

#define XS_BLOCK_SIZE                 (LIST_DATA_SIZE + 135)

#define XS_VHI                        (LIST_DATA_SIZE + 0)
#define XS_VHI_IDX                    (LIST_DATA_SIZE + 1)
#define XS_BHI                        (LIST_DATA_SIZE + 2)
#define XS_BHI_IDX                    (LIST_DATA_SIZE + 3)
#define XS_BOR                        (LIST_DATA_SIZE + 4)
#define XS_BOR_IDX                    (LIST_DATA_SIZE + 5)
#define XS_CR                         (LIST_DATA_SIZE + 6)
#define XS_CR_IDX                     (LIST_DATA_SIZE + 7)
#define XS_TFU                        (LIST_DATA_SIZE + 8)
#define XS_TFU_IDX                    (LIST_DATA_SIZE + 9)
#define XS_VOI                        (LIST_DATA_SIZE + 10)
#define XS_VOI_IDX                    (LIST_DATA_SIZE + 11)
#define XS_TMO                        (LIST_DATA_SIZE + 12)
#define XS_TMO_IDX                    (LIST_DATA_SIZE + 13)
#define XS_EXP                        (LIST_DATA_SIZE + 14)
#define XS_EXP_IDX                    (LIST_DATA_SIZE + 15)
#define XS_RHO                        (LIST_DATA_SIZE + 16)
#define XS_RHO_IDX                    (LIST_DATA_SIZE + 17)
#define XS_SIGA1                      (LIST_DATA_SIZE + 20)
#define XS_SIGA2                      (LIST_DATA_SIZE + 21)
#define XS_NUSF1                      (LIST_DATA_SIZE + 22)
#define XS_NUSF2                      (LIST_DATA_SIZE + 23)
#define XS_DIFF1                      (LIST_DATA_SIZE + 24)
#define XS_DIFF2                      (LIST_DATA_SIZE + 25)
#define XS_SIG11                      (LIST_DATA_SIZE + 26)
#define XS_SIG12                      (LIST_DATA_SIZE + 27)
#define XS_SIG22                      (LIST_DATA_SIZE + 28)
#define XS_SIG21                      (LIST_DATA_SIZE + 29)
#define XS_EPSF1                      (LIST_DATA_SIZE + 30)
#define XS_EPSF2                      (LIST_DATA_SIZE + 31)
#define XS_SIGF1                      (LIST_DATA_SIZE + 32)
#define XS_SIGF2                      (LIST_DATA_SIZE + 33)
#define XS_SDF1W                      (LIST_DATA_SIZE + 34) /* peräkkäin */
#define XS_SDF1S                      (LIST_DATA_SIZE + 35)
#define XS_SDF1E                      (LIST_DATA_SIZE + 36)
#define XS_SDF1N                      (LIST_DATA_SIZE + 37)
#define XS_SDF2W                      (LIST_DATA_SIZE + 38) /* peräkkäin */
#define XS_SDF2S                      (LIST_DATA_SIZE + 39)
#define XS_SDF2E                      (LIST_DATA_SIZE + 40)
#define XS_SDF2N                      (LIST_DATA_SIZE + 41)
#define XS_CD1SW                      (LIST_DATA_SIZE + 42) /* peräkkäin */
#define XS_CD1SE                      (LIST_DATA_SIZE + 43)
#define XS_CD1NE                      (LIST_DATA_SIZE + 44)
#define XS_CD1NW                      (LIST_DATA_SIZE + 45)
#define XS_CD2SW                      (LIST_DATA_SIZE + 46) /* peräkkäin */
#define XS_CD2SE                      (LIST_DATA_SIZE + 47)
#define XS_CD2NE                      (LIST_DATA_SIZE + 48)
#define XS_CD2NW                      (LIST_DATA_SIZE + 49)
#define XS_SIGI1                      (LIST_DATA_SIZE + 50)
#define XS_SIGI2                      (LIST_DATA_SIZE + 51)
#define XS_SIGI3                      (LIST_DATA_SIZE + 52)
#define XS_SIGX1                      (LIST_DATA_SIZE + 53)
#define XS_SIGX2                      (LIST_DATA_SIZE + 54)
#define XS_SIGX3                      (LIST_DATA_SIZE + 55)
#define XS_SIGP1                      (LIST_DATA_SIZE + 56)
#define XS_SIGP2                      (LIST_DATA_SIZE + 57)
#define XS_SIGP3                      (LIST_DATA_SIZE + 58)
#define XS_SIGS1                      (LIST_DATA_SIZE + 59)
#define XS_SIGS2                      (LIST_DATA_SIZE + 60)
#define XS_SIGS3                      (LIST_DATA_SIZE + 61)
#define XS_YLD_I1                     (LIST_DATA_SIZE + 62)
#define XS_YLD_I2                     (LIST_DATA_SIZE + 63)
#define XS_YLD_XE1                    (LIST_DATA_SIZE + 64)
#define XS_YLD_XE2                    (LIST_DATA_SIZE + 65)
#define XS_YLD_PM1                    (LIST_DATA_SIZE + 66)
#define XS_YLD_PM2                    (LIST_DATA_SIZE + 67)
#define XS_YLD_SM1                    (LIST_DATA_SIZE + 68)
#define XS_YLD_SM2                    (LIST_DATA_SIZE + 69)
#define XS_MACRO_XE1                  (LIST_DATA_SIZE + 70)
#define XS_MACRO_XE2                  (LIST_DATA_SIZE + 71)
#define XS_MACRO_SM1                  (LIST_DATA_SIZE + 72)
#define XS_MACRO_SM2                  (LIST_DATA_SIZE + 73)
#define XS_BETA                       (LIST_DATA_SIZE + 74) /* peräkkäin */
#define XS_BETA1                      (LIST_DATA_SIZE + 75)
#define XS_BETA2                      (LIST_DATA_SIZE + 76)
#define XS_BETA3                      (LIST_DATA_SIZE + 77)
#define XS_BETA4                      (LIST_DATA_SIZE + 78)
#define XS_BETA5                      (LIST_DATA_SIZE + 79)
#define XS_BETA6                      (LIST_DATA_SIZE + 80)
#define XS_BETA7                      (LIST_DATA_SIZE + 81)
#define XS_BETA8                      (LIST_DATA_SIZE + 82)
#define XS_INVV1                      (LIST_DATA_SIZE + 83)
#define XS_INVV2                      (LIST_DATA_SIZE + 84)
#define XS_LBDA                       (LIST_DATA_SIZE + 85) /* peräkkäin */
#define XS_LBDA1                      (LIST_DATA_SIZE + 86)
#define XS_LBDA2                      (LIST_DATA_SIZE + 87)
#define XS_LBDA3                      (LIST_DATA_SIZE + 88)
#define XS_LBDA4                      (LIST_DATA_SIZE + 89)
#define XS_LBDA5                      (LIST_DATA_SIZE + 90)
#define XS_LBDA6                      (LIST_DATA_SIZE + 91)
#define XS_LBDA7                      (LIST_DATA_SIZE + 92)
#define XS_LBDA8                      (LIST_DATA_SIZE + 93)
#define XS_FLUX1                      (LIST_DATA_SIZE + 94)
#define XS_FLUX2                      (LIST_DATA_SIZE + 95)
#define XS_XE_FLX                     (LIST_DATA_SIZE + 96)
#define XS_SM_FLX                     (LIST_DATA_SIZE + 97)
#define XS_SDF11                      (LIST_DATA_SIZE + 98) /* peräkkäin */
#define XS_SDF12                      (LIST_DATA_SIZE + 99)
#define XS_SDF13                      (LIST_DATA_SIZE + 100)
#define XS_SDF14                      (LIST_DATA_SIZE + 101)
#define XS_SDF15                      (LIST_DATA_SIZE + 102)
#define XS_SDF16                      (LIST_DATA_SIZE + 103)
#define XS_SDF21                      (LIST_DATA_SIZE + 104)
#define XS_SDF22                      (LIST_DATA_SIZE + 105)
#define XS_SDF23                      (LIST_DATA_SIZE + 106)
#define XS_SDF24                      (LIST_DATA_SIZE + 107)
#define XS_SDF25                      (LIST_DATA_SIZE + 108)
#define XS_SDF26                      (LIST_DATA_SIZE + 109)
#define XS_CDF11                      (LIST_DATA_SIZE + 110) /* peräkkäin */
#define XS_CDF12                      (LIST_DATA_SIZE + 111)
#define XS_CDF13                      (LIST_DATA_SIZE + 112)
#define XS_CDF14                      (LIST_DATA_SIZE + 113)
#define XS_CDF15                      (LIST_DATA_SIZE + 114)
#define XS_CDF16                      (LIST_DATA_SIZE + 115)
#define XS_CDF21                      (LIST_DATA_SIZE + 116)
#define XS_CDF22                      (LIST_DATA_SIZE + 117)
#define XS_CDF23                      (LIST_DATA_SIZE + 118)
#define XS_CDF24                      (LIST_DATA_SIZE + 119)
#define XS_CDF25                      (LIST_DATA_SIZE + 120)
#define XS_CDF26                      (LIST_DATA_SIZE + 121)
#define XS_TRANSP1                    (LIST_DATA_SIZE + 122)
#define XS_TRANSP2                    (LIST_DATA_SIZE + 123)
#define XS_USED                       (LIST_DATA_SIZE + 124)
#define XS_TESTED                     (LIST_DATA_SIZE + 125)
#define XS_PTR_GCU                    (LIST_DATA_SIZE + 126)
#define XS_PTR_HIS                    (LIST_DATA_SIZE + 127)
#define XS_PTR_LIB                    (LIST_DATA_SIZE + 128)
#define XS_PARCS_STA_VAR              (LIST_DATA_SIZE + 129)
#define XS_PARCS_STA_IDX              (LIST_DATA_SIZE + 130)
#define XS_CHIP1                      (LIST_DATA_SIZE + 131)
#define XS_CHIP2                      (LIST_DATA_SIZE + 132)
#define XS_CHID1                      (LIST_DATA_SIZE + 133)
#define XS_CHID2                      (LIST_DATA_SIZE + 134)

/*****************************************************************************/
