/*****************************************************************************/
/*                                                                           */
/* AresFit : main.c                                                          */
/*                                                                           */
/* Created:       2014/08/25 (JLe)                                           */
/* Last modified: 2016/03/06 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: Main program file                                            */
/*                                                                           */
/* Comments:                                                                 */
/*                                                                           */
/*****************************************************************************/

#include "header.h"
#include "locations.h"

#define FUNCTION_NAME "Main:"

/*****************************************************************************/

int main(int argc, char** argv)
{
  long np, n, lib, his, ptr, flg;
  char tmpstr[MAX_STR], c;
  FILE *fp;

  /***************************************************************************/

  /***** Init stuff **********************************************************/

  /* Errors to stderr */

  err = stderr;

  /* Reset pointers */

  RDB = NULL;
  WDB = NULL;
  ASCII = NULL;

  /* Allocate memory for main data block */

  ReallocMem(DATA_ARRAY, DATA_LAST_VALUE);

  /* Put read-only pointer */

  RDB = (const double *)WDB;

 /* Allocate memory for ASCII data block to simplify error testing */

  ASCII = (char *)Mem(MEM_ALLOC, (long)(VALID_PTR + 1), sizeof(char)); 
  WDB[DATA_ASCII_DATA_SIZE] = (double)(VALID_PTR + 1);

  /* Print */

  printf("\n%s %s (%s) -- Contact: %s\n\n", CODE_NAME, CODE_VERSION, 
	 CODE_DATE, CODE_AUTHOR);

  /* Mode */

  WDB[DATA_MODE] = (double)MODE_DEP;

  /***************************************************************************/

  /***** Read main input file ************************************************/

  /* Avoid compiler warning */

  fp = NULL;

  /* Open file for reading */

  if (argc < 2)
    Die(NULL, "Input file not given");
  else
    fp = OpenFile(argv[1], "r");

  /* Read file */

  while (fscanf(fp, "%s", tmpstr) != EOF)
    {
      /* Check key words */
  
      if (!strcasecmp(tmpstr, "mode"))
	{
	  /* Read code */

	  if (fscanf(fp, "%s", tmpstr) == EOF)
	    Die(NULL, "Input error: Unexpected EOF");

	  /* Check */

	  if (!strcasecmp(tmpstr, "ares"))
	    WDB[DATA_CODE] = (double)CODE_ARES;
	  else if (!strcasecmp(tmpstr, "apros"))
	    WDB[DATA_CODE] = (double)CODE_APROS;
	  else if (!strcasecmp(tmpstr, "parcs"))
	    WDB[DATA_CODE] = (double)CODE_PARCS;
	  else
	    Die(NULL, "Invalid code '%s'", tmpstr);

	  /* Read mode */

	  if (fscanf(fp, "%s", tmpstr) == EOF)
	    Die(NULL, "Input error: Unexpected EOF");

	  /* Check */

	  if (!strcasecmp(tmpstr, "hzp"))
	    WDB[DATA_MODE] = (double)MODE_HZP;
	  else if (!strcasecmp(tmpstr, "hfp"))
	    WDB[DATA_MODE] = (double)MODE_HFP;
	  else if (!strcasecmp(tmpstr, "dep"))
	    WDB[DATA_MODE] = (double)MODE_DEP;
	  else if (!strcasecmp(tmpstr, "base"))
	    WDB[DATA_MODE] = (double)MODE_BAS;
	  else
	    Die(NULL, "Invalid mode '%s'", tmpstr);
	}
      else if (!strcasecmp(tmpstr, "lib"))
	{
	  /* Create new library */
	      
	  lib = NewItem(DATA_PTR_LIB0, LIB_BLOCK_SIZE);

	  /* Read name */

	  if (fscanf(fp, "%s", tmpstr) == EOF)
	    Die(NULL, "Input error: Unexpected EOF");
	      
	  WDB[lib + LIB_PTR_NAME] = PutText(tmpstr);

	  /* Check code */

	  if ((long)RDB[DATA_CODE] == CODE_APROS)
	    {
	      /* Read title (first entry reads newline) */

	      if (fgets(tmpstr, 82, fp) == NULL)
		Die(NULL, "Input error: Unexpected EOF");

	      if (fgets(tmpstr, 82, fp) == NULL)
		Die(NULL, "Input error: Unexpected EOF");

	      /* Replace newline */

	      tmpstr[strlen(tmpstr) - 1] = '\0';
	      
	      /* Put title */

	      WDB[lib + LIB_PTR_TIT] = PutText(tmpstr);

	      /* Read MATER data */

	      WDB[lib + LIB_MATER_MNUM] = (double)ReadI(fp);
	      WDB[lib + LIB_MATER_UMASS] = ReadF(fp);

	      /* Read NOMCON data */

	      WDB[lib + LIB_NOMCON_PNOM] = ReadF(fp);
	      WDB[lib + LIB_NOMCON_KXE] = (double)ReadI(fp);
	      WDB[lib + LIB_NOMCON_KSM] = (double)ReadI(fp);
	      WDB[lib + LIB_NOMCON_TFNOM] = ReadF(fp);
	      WDB[lib + LIB_NOMCON_TMNOM] = ReadF(fp);
	      WDB[lib + LIB_NOMCON_TINOM] = ReadF(fp);
	      WDB[lib + LIB_NOMCON_CBNOM] = ReadF(fp);
	      WDB[lib + LIB_NOMCON_DMNOM] = ReadF(fp);
	      WDB[lib + LIB_NOMCON_DINOM] = ReadF(fp);
	      WDB[lib + LIB_NOMCON_ALFA] = ReadF(fp);
	      WDB[lib + LIB_NOMCON_MODC] = (double)ReadI(fp);
	      WDB[lib + LIB_NOMCON_NHET] = (double)ReadI(fp);

	      WDB[lib + LIB_FITCO_NXE] = (double)ReadI(fp);
	      WDB[lib + LIB_FITCO_NSM] = (double)ReadI(fp);

	      ptr = ReallocMem(DATA_ARRAY, 7*5);

	      WDB[lib + LIB_FITCO_PTR_DOPPLER] = (double)ptr;
	      for (n = 0; n < 7; n++)
		WDB[ptr++] = (long)ReadI(fp);

	      WDB[lib + LIB_FITCO_PTR_BORON] = (double)ptr;
	      for (n = 0; n < 7; n++)
		WDB[ptr++] = (long)ReadI(fp);

	      WDB[lib + LIB_FITCO_PTR_MOD_DENS_TEMP] = (double)ptr;
	      for (n = 0; n < 7; n++)
		WDB[ptr++] = (long)ReadI(fp);

	      WDB[lib + LIB_FITCO_PTR_MOD_DENS] = (double)ptr;
	      for (n = 0; n < 7; n++)
		WDB[ptr++] = (long)ReadI(fp);

	      WDB[lib + LIB_FITCO_PTR_MOD_TEMP] = (double)ptr;
	      for (n = 0; n < 7; n++)
		WDB[ptr++] = (long)ReadI(fp);
	    }
	  else if ((long)RDB[DATA_CODE] == CODE_ARES)
	    {
	      /* Read nominal values */
	      
	      WDB[lib + LIB_NOM_VOI] = ReadF(fp);
	      WDB[lib + LIB_NOM_TFU] = ReadF(fp);
	      WDB[lib + LIB_NOM_TMO] = ReadF(fp);
	    }
	  else if ((long)RDB[DATA_CODE] == CODE_PARCS)
	    {
	      /* Read number of SDF's and CDF's */

	      WDB[lib + LIB_INPUT_NSDF] = ReadI(fp);
	      WDB[lib + LIB_INPUT_NCDF] = ReadI(fp);

	      /* Read flag array */

	      flg = 0;
	      for (n = 0; n < 15; n++)
		{
		  if (fscanf(fp, " %c", &c) == EOF)
		    Die(NULL, "Input error: Unexpected EOF");
		  else if ((c == 'T') || (c == 't'))
		    flg = flg + (long)pow(2,n);
		  else if ((c != 'F') && (c != 'f'))
		    Die(NULL, "Invalid flag '%c'", c);
		}

	      /* Check that derivative flag is not set */

	      if (flg & PARCS_FLAG_LDER)
		Die(NULL, "Partial derivative flag cannot be set");

	      /* Store value */

	      WDB[lib + LIB_PARCS_FLAGS] = (double)flg;
	      
	      /* Read XS set series number */

	      if (fscanf(fp, "%s", tmpstr) == EOF)
		Die(NULL, "Input error: Unexpected EOF");
	      
	      WDB[lib + LIB_PARCS_PTR_ID] = PutText(tmpstr);

	      /* Read pitch, heavy metal density and saturated moderator */
	      /* density */

	      WDB[lib + LIB_PARCS_PITCH] = ReadF(fp);
	      WDB[lib + LIB_PARCS_IHMD] = ReadF(fp);
	      WDB[lib + LIB_PARCS_DSAT] = ReadF(fp);
	    }
	  else
	    Die(NULL, "Mode must be set before defining libraries");

	  /* Read number of histories */

	  np = ReadI(fp);

	  /* Read files */
	  
	  for (n = 0; n < np; n++)
	    {
	      /* New history */

	      his = NewItem(lib + LIB_PTR_HIS, HIS_BLOCK_SIZE);

	      /* Read file name */

	      if (fscanf(fp, "%s", tmpstr) == EOF)
		Die(NULL, "Input error: Unexpected EOF");
	      else
		WDB[his + HIS_PTR_FILE] = PutText(tmpstr);
	    }
	}
      else
	Die(NULL, "Input error: Invalid parameter '%s'", tmpstr);
    }

  /* Read data */

  ReadData();

  /* Process data */
  
  if ((long)RDB[DATA_CODE] == CODE_ARES)
    ProcessARES();
  else if ((long)RDB[DATA_CODE] == CODE_APROS)
    ProcessApros();
  else if ((long)RDB[DATA_CODE] == CODE_PARCS)
    ProcessPARCS();

  /* Close file */

  fclose(fp);

  /***************************************************************************/

  /***** Free allocated memory ***********************************************/

  /* Free data arrays */

  if (WDB != NULL)
    Mem(MEM_FREE, WDB);

  if (ASCII != NULL)
    Mem(MEM_FREE, ASCII);

  /***************************************************************************/

  /* Exit */
  
  return 0;
}

/*****************************************************************************/
