/*****************************************************************************/
/*                                                                           */
/* AresFit : mem.c                                                           */
/*                                                                           */
/* Created:       2014/08/25 (JLe)                                           */
/* Last modified: 2014/08/25 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: Calls calloc(), realloc() or free()                          */
/*                                                                           */
/* Comments:                                                                 */
/*                                                                           */
/*****************************************************************************/

#include "header.h"
#include "locations.h"

#define FUNCTION_NAME "Mem:"

/*****************************************************************************/

void *Mem(long mode, ...)
{
  void *ptr0, *ptr1;
  size_t nelem, size;
  va_list argp;
  va_start (argp, mode);
  
  /* Set null return pointer */
  
  ptr1 = NULL;
  
  /* Check mode */
  
  if (mode == MEM_ALLOC)
    {
      /* Get arguments */
      
      nelem = va_arg(argp, size_t);
      size = va_arg(argp, size_t);
      
      /* Check sizes */
      
      if (nelem < 1)
	Die(FUNCTION_NAME, "Zero or negative block size (calloc)");
      if (size < 1)
	Die(FUNCTION_NAME, "Zero or negative element size (calloc)");	
      
      /* Allocate new block */
      
      if ((ptr1 = calloc(nelem, size)) == NULL)
	Die(FUNCTION_NAME, "Memory allocation failed (calloc, %ld, %ld)",
	    nelem, size);
    }
  else if (mode == MEM_REALLOC)
    {
      /* Get arguments */
      
      ptr0 = va_arg(argp, void *);
      size = va_arg(argp, size_t);
      
      /* Check size */

      if (size < 1)
	Die(FUNCTION_NAME, "Zero or negative block size (realloc)");
      
      /* Adjust size of existing block */
      
      if ((ptr1 = realloc(ptr0, size)) == NULL)
	Die(FUNCTION_NAME, "Memory allocation failed (realloc, size = %ld)",
	    size);
      }
  else if (mode == MEM_FREE)
    {
      /* Get arguments */
      
      ptr0 = va_arg(argp, void *);
      
      /* Check pointer */
      
      if (ptr0 == NULL)
	Die(FUNCTION_NAME, "Null pointer (free)");
      
      /* Free memory */
      
      free(ptr0);
    }
  else
    Die(FUNCTION_NAME, "Invalid mode");
  
  /* Return pointer */

  return ptr1;
}

/*****************************************************************************/
