/*****************************************************************************/
/*                                                                           */
/* AresFit : processapros.c                                                  */
/*                                                                           */
/* Created:       2014/12/15 (JLe)                                           */
/* Last modified: 2016/11/01 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: Main data processing routine for APROS                       */ 
/*                                                                           */
/* Comments:                                                                 */
/*                                                                           */
/*****************************************************************************/

#include "header.h"
#include "locations.h"

#define FUNCTION_NAME "ProcessApros:"

/*****************************************************************************/

void ProcessApros()
{
  long lib, xsp, his, ncr, nvh, nbu, ntf, nvo, ntm, nbo, nbh, ptr, nde;
  long i, j, k, mode, nhet, nmat;
  double VHI[3], BHI[3], BOR[3], CR[3], TFU[3], VOI[3], TMO[3], EXP[100];
  double BU[100], RHO[3], val, x[3], y[3], *p;
  FILE *fp;
  char tmpstr[MAX_STR];

  /* Get mode */
    
  mode = (long)RDB[DATA_MODE];

  /* Reset material index */

  nmat = 0;

   /* Loop over libraries */

  lib = (long)RDB[DATA_PTR_LIB0];
  while (lib > VALID_PTR)
    {
      /* Print */
      
      printf("Processing library '%s'...\n", GetText(lib + LIB_PTR_NAME));

      /************************************************************************/

      /***** Print header *****************************************************/

      /* Open file */

      sprintf(tmpstr, "%s.xs", GetText(lib + LIB_PTR_NAME));
      fp = OpenFile(tmpstr, "w");

      /***********************************************************************/

      /***** Collect variables into vectors **********************************/

      /* Reset maximum number of branches */

      ncr = 0; 
      nvh = 0; 
      nbu = 0;
      ntf = 0;
      nvo = 0;
      ntm = 0;
      nbo = 0;
      nbh = 0;
      nde = 0;
      
      /* Loop over histories */
      
      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  /* Loop over data */
	  
	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Get parameter values */
	      
	      i = (long)RDB[xsp + XS_VHI_IDX];
	      VHI[i] = RDB[xsp + XS_VHI];
	      
	      if (i + 1 > nvh)
		nvh = i + 1;
	      
	      i = (long)RDB[xsp + XS_BHI_IDX];
	      BHI[i] = RDB[xsp + XS_BHI];
	      
	      if (i + 1 > nbh)
		nbh = i + 1;
	      
	      i = (long)RDB[xsp + XS_BOR_IDX];
	      BOR[i] = RDB[xsp + XS_BOR];
	      
	      if (i + 1 > nbo)
		nbo = i + 1;
	      
	      i = (long)RDB[xsp + XS_TFU_IDX];
	      TFU[i] = RDB[xsp + XS_TFU];
	      
	      if (i + 1 > ntf)
		ntf = i + 1;
	      
	      i = (long)RDB[xsp + XS_VOI_IDX];
	      VOI[i] = RDB[xsp + XS_VOI];
	      
	      if (i + 1 > nvo)
		nvo = i + 1;
	      
	      i = (long)RDB[xsp + XS_TMO_IDX];
	      TMO[i] = RDB[xsp + XS_TMO];
	      
	      if (i + 1 > ntm)
		ntm = i + 1;

	      i = (long)RDB[xsp + XS_RHO_IDX];
	      RHO[i] = RDB[xsp + XS_RHO];

	      if (i + 1 > nde)
		nde = i + 1;
	      
	      i = (long)RDB[xsp + XS_EXP_IDX];
	      
	      if (i > 99)
		Die(FUNCTION_NAME, "Increase vector size");
	      else
		EXP[i] = RDB[xsp + XS_EXP];
	      
	      if (i + 1 > nbu)
		nbu = i + 1;
	      
	      i = (long)RDB[xsp + XS_CR_IDX];
	      CR[i] = RDB[xsp + XS_CR];
	      
	      if (i + 1 > ncr)
		ncr = i + 1;
	      
	      /* Next */
	      
	      xsp = NextItem(xsp);
	    }
	  
	  /* Next history */
	  
	  his = NextItem(his);
	}

      /* Make vector of printed burnups */

      for (i = 0; i < nbu; i++)
	BU[i] = EXP[i];

      /* Add one point if not in burnup mode */
      
      if (mode < MODE_DEP)
	{
	  /* Check number of points */

	  if (nbu != 1)
	    Die(FUNCTION_NAME, "WTF?");
	  /*else
	    nbu++;*/

	  /* Put values */

	  /*EXP[1] = 0.0;
	  BU[1] = 100.0;*/
	}

      /* Check parametrization */

      if (mode > MODE_BAS)
	{
	  /* Boron */

	  if (nbo < 2)
	    Die(NULL, "Missing boron branches or variable BOR");
	}
      
      if (mode > MODE_HZP)
	{
	  /* Fuel temperature */

	  if (ntf < 2)
	    Die(NULL, "Missing fuel temperature branches or variable TFU");

	  /* Moderator density */

	  if (nde < 2)
	    Die(NULL, "Missing moderator density branches or variable RHO");
	}

      /* Print (debug) */
      
      if (1 != 2)
	{
	  for (i = 0; i < nvh; i++)
	    printf("VHI = %E\n", VHI[i]);
	  
	  for (i = 0; i < nbh; i++)
	    printf("BHI = %E\n", BHI[i]);
	  
	  for (i = 0; i < nbo; i++)
	    printf("BOR = %E\n", BOR[i]);
	  
	  for (i = 0; i < ntf; i++)
	    printf("TFU = %E\n", TFU[i]);
	  
	  for (i = 0; i < nvo; i++)
	    printf("VOI = %E\n", VOI[i]);
	  
	  for (i = 0; i < ntm; i++)
	    printf("TMO = %E\n", TMO[i]);

	  for (i = 0; i < nde; i++)
	    printf("RHO = %E\n", RHO[i]);
	      
	  for (i = 0; i < nbu; i++)
	    printf("EXP = %E\n", EXP[i]);
	  
	  for (i = 0; i < ncr; i++)
	    printf("CR = %E\n", CR[i]);
	}

      /* Put nominal values */

      WDB[lib + LIB_NOMCON_TFNOM] = TFU[0];
      WDB[lib + LIB_NOMCON_TMNOM] = TMO[0];
      WDB[lib + LIB_NOMCON_TINOM] = TMO[0];
      WDB[lib + LIB_NOMCON_CBNOM] = BOR[0];
      WDB[lib + LIB_NOMCON_DMNOM] = RHO[0];
      WDB[lib + LIB_NOMCON_DINOM] = RHO[0];

      /***********************************************************************/

      /***** Print header ****************************************************/

      /* Print CASE data */

      if ((long)RDB[lib + LIB_PTR_TIT] < VALID_PTR)
	Die(NULL, "Title not set");
      else
	fprintf(fp, "/CASE/ '%s'\n", GetText(lib + LIB_PTR_TIT));

      /* Print MATER data */

      fprintf(fp, "/MATER/    %3ld %11.4E     1\n", 
	      (long)RDB[lib + LIB_MATER_MNUM], 
	      RDB[lib + LIB_MATER_UMASS]);

      /* Print NOMCON data */

      fprintf(fp, "/NOMCON/  PNOM=%7.3f  KXE=%ld KSM=%ld  TFNOM=%6.1f ",
	      RDB[lib + LIB_NOMCON_PNOM], (long)RDB[lib + LIB_NOMCON_KXE],
	      (long)RDB[lib + LIB_NOMCON_KSM], TFU[0] - 273.15);

      fprintf(fp, "TMNOM=%6.1f TINOM=%6.1f\n          CBNOM=%6.1f ",
	      TMO[0] - 273.15, TMO[0] - 273.15, BOR[0]);

      fprintf(fp, "DMNOM=%7.5f DINOM=%7.5f\n          ALFA=%6.3f ",
	      RDB[lib + LIB_NOMCON_DMNOM], RDB[lib + LIB_NOMCON_DINOM],
	      RDB[lib + LIB_NOMCON_ALFA]);

      fprintf(fp, "MODC=%2ld NHET=%2ld\n",
	      (long)RDB[lib + LIB_NOMCON_MODC], 
	      (long)RDB[lib + LIB_NOMCON_NHET]);

      /* Print FITCO data */
      
      fprintf(fp, "/FITCO/   %ld  %ld    ", (long)RDB[lib + LIB_FITCO_NXE], 
	      (long)RDB[lib + LIB_FITCO_NSM]);

      ptr = (long)RDB[lib + LIB_FITCO_PTR_DOPPLER];
      CheckPointer(FUNCTION_NAME, "(ptr)", DATA_ARRAY, ptr);
      
      for (i = 0; i < 5; i++)
	{
	  if (i > 0)
	    fprintf(fp, "                  ");

	  for (j = 0; j < 7; j++)
	    fprintf(fp, "%2ld  ", (long)RDB[ptr++]);

	  fprintf(fp, "\n");
	}

      /***********************************************************************/
    
      /***** Print base values ***********************************************/

      /* Get number of discontinuity factors */

      nhet = (long)RDB[lib + LIB_NOMCON_NHET];
   
      /* Loop over burnups (HUOM: tiheys lämpötilan paikalla) */

      for (k = 0; k < nbu; k++)
	{
	  fprintf(fp, "/DIFCON/    %8.1f", 1000*BU[k]);

	  val =  GetXS(lib, XS_DIFF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 0.0);

	  val =  GetXS(lib, XS_SIGA1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 0.0);

	  val =  GetXS(lib, XS_NUSF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 1E-16);

	  val =  GetXS(lib, XS_SIGF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 1E-16);

	  val =  GetXS(lib, XS_SIG12, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E\n        ", val);
	  else
	    fprintf(fp, " %11.4E\n        ", 0.0);

	  val =  GetXS(lib, XS_DIFF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 0.0);

	  val =  GetXS(lib, XS_SIGA2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 0.0);

	  val =  GetXS(lib, XS_NUSF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 1E-16);

	  val =  GetXS(lib, XS_SIGF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 1E-16);

	  val =  GetXS(lib, XS_EPSF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 1E-16);

	  if (nhet == 1)
	    {
	      fprintf(fp, "\n        ");
	      fprintf(fp, " %11.4E", GetXS(lib, XS_SDF1W, VHI[0], BHI[0], 
					   BOR[0], CR[0], TFU[0], VOI[0], 
					   RHO[0], EXP[k]));
	      fprintf(fp, " %11.4E", GetXS(lib, XS_SDF2W, VHI[0], BHI[0], 
					   BOR[0], CR[0], TFU[0], VOI[0], 
					   RHO[0], EXP[k]));
	    }
	  else if (nhet == 2)
	    {
	      fprintf(fp, "\n        ");
	      fprintf(fp, " %11.4E", GetXS(lib, XS_SDF11, VHI[0], BHI[0], 
					   BOR[0], CR[0], TFU[0], VOI[0], 
					   RHO[0], EXP[k]));
	      fprintf(fp, " %11.4E", GetXS(lib, XS_SDF21, VHI[0], BHI[0], 
					   BOR[0], CR[0], TFU[0], VOI[0], 
					   RHO[0], EXP[k]));
	    }
	  else if (nhet == 4)
	    {
	      fprintf(fp, "\n        ");

	      val =  GetXS(lib, XS_SDF1W, VHI[0], BHI[0], BOR[0], CR[0], 
			   TFU[0], VOI[0], RHO[0], EXP[k]);
	      if (val > 0.0)
		fprintf(fp, " %11.4E", val);
	      else
		fprintf(fp, " %11.4E", 1.0);

	      val =  GetXS(lib, XS_SDF1S, VHI[0], BHI[0], BOR[0], CR[0], 
			   TFU[0], VOI[0], RHO[0], EXP[k]);
	      if (val > 0.0)
		fprintf(fp, " %11.4E", val);
	      else
		fprintf(fp, " %11.4E", 1.0);

	      val =  GetXS(lib, XS_SDF1E, VHI[0], BHI[0], BOR[0], CR[0], 
			   TFU[0], VOI[0], RHO[0], EXP[k]);
	      if (val > 0.0)
		fprintf(fp, " %11.4E", val);
	      else
		fprintf(fp, " %11.4E", 1.0);

	      val =  GetXS(lib, XS_SDF1N, VHI[0], BHI[0], BOR[0], CR[0], 
			   TFU[0], VOI[0], RHO[0], EXP[k]);
	      if (val > 0.0)
		fprintf(fp, " %11.4E", val);
	      else
		fprintf(fp, " %11.4E", 1.0);

	      val =  GetXS(lib, XS_SDF2W, VHI[0], BHI[0], BOR[0], CR[0], 
			   TFU[0], VOI[0], RHO[0], EXP[k]);
	      if (val > 0.0)
		fprintf(fp, " %11.4E", val);
	      else
		fprintf(fp, " %11.4E", 1.0);

	      val =  GetXS(lib, XS_SDF2S, VHI[0], BHI[0], BOR[0], CR[0], 
			   TFU[0], VOI[0], RHO[0], EXP[k]);
	      if (val > 0.0)
		fprintf(fp, " %11.4E", val);
	      else
		fprintf(fp, " %11.4E", 1.0);

	      fprintf(fp, "\n        ");

	      val =  GetXS(lib, XS_SDF2E, VHI[0], BHI[0], BOR[0], CR[0], 
			   TFU[0], VOI[0], RHO[0], EXP[k]);
	      if (val > 0.0)
		fprintf(fp, " %11.4E", val);
	      else
		fprintf(fp, " %11.4E", 1.0);

	      val =  GetXS(lib, XS_SDF2N, VHI[0], BHI[0], BOR[0], CR[0], 
			   TFU[0], VOI[0], RHO[0], EXP[k]);
	      if (val > 0.0)
		fprintf(fp, " %11.4E", val);
	      else
		fprintf(fp, " %11.4E", 1.0);
	    }
	  else
	    Die(FUNCTION_NAME, "NHET = %ld\n", nhet);

	  fprintf(fp, "\n        ");

	  val =  GetXS(lib, XS_INVV1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 0.0);

	  val =  GetXS(lib, XS_INVV2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 0.0);

	  fprintf(fp, "\n        ");

	  val =  GetXS(lib, XS_BETA1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 0.0);

	  val =  GetXS(lib, XS_BETA2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 0.0);

	  val =  GetXS(lib, XS_BETA3, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 0.0);

	  val =  GetXS(lib, XS_BETA4, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 0.0);

	  val =  GetXS(lib, XS_BETA5, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 0.0);

	  val =  GetXS(lib, XS_BETA6, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
		       VOI[0], RHO[0], EXP[k]);
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 0.0);
	  
	  fprintf(fp, "\n");
	}

      /***********************************************************************/
      
      /***** Print poison data ***********************************************/

      /* HUOM: Nää on jotain ihan kummallista */

      /* Xenon data */

      for (k = 0; k < nbu; k++)
	{
	  /* APROS needs non-zero values, so print number of the beast */
	  /* in place of zeros */

	  fprintf(fp, "/XENON/     %8.1f", 1000*BU[k]);

	  /* Yield of Xe-135 + I-135 */

	  val =  GetXS(lib, XS_SIGX3, VHI[0], BHI[0], BOR[0], 
		       CR[0], TFU[0], VOI[0], RHO[0], EXP[k])
	    +
	    GetXS(lib, XS_SIGI3, VHI[0], BHI[0], BOR[0], 
		  CR[0], TFU[0], VOI[0], RHO[0], EXP[k]);
	  
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 666.0);

	  /* Xenon flux */

	  val = GetXS(lib, XS_XE_FLX, VHI[0], BHI[0], BOR[0], 
		      CR[0], TFU[0], VOI[0], RHO[0], EXP[k]);

	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 666.0);
	  /*
	  fprintf(fp, " %11.4E", GetXS(lib, XS_SIGI3, VHI[0], BHI[0], BOR[0], 
				       CR[0], TFU[0], VOI[0], RHO[0], EXP[k]));
	  */
	  fprintf(fp, "\n");
	}

      /* Samarium data */

      for (k = 0; k < nbu; k++)
	{
	  fprintf(fp, "/SAMAR/     %8.1f", 1000*BU[k]);

	  /* Samarium yield */

	  val = GetXS(lib, XS_SIGS3, VHI[0], BHI[0], BOR[0], 
		      CR[0], TFU[0], VOI[0], RHO[0], EXP[k])
	    +
	    GetXS(lib, XS_SIGP3, VHI[0], BHI[0], BOR[0], 
		  CR[0], TFU[0], VOI[0], RHO[0], EXP[k]);
	  
	  /* Force to zero on zero burnup */

	  if (k == 0)
	    fprintf(fp, " %11.4E", 0.0);
	  else if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 666.0);

	  /* Samarium flux */

	  val = GetXS(lib, XS_SM_FLX, VHI[0], BHI[0], BOR[0], 
		      CR[0], TFU[0], VOI[0], RHO[0], EXP[k]);
	  
	  if (val > 0.0)
	    fprintf(fp, " %11.4E", val);
	  else
	    fprintf(fp, " %11.4E", 666.0);

	  fprintf(fp, "\n");
	}

      /***********************************************************************/
      
      /***** Print feedback coefficients *************************************/
      
      /* Get temperature vector */

      x[0] = 0.0;
      x[1] = sqrt(TFU[1]) - sqrt(TFU[0]);
      x[2] = sqrt(TFU[2]) - sqrt(TFU[0]);

      /* Loop over burnups */

      if (ntf == 3)
	for (k = 0; k < nbu; k++)
	  {
	    fprintf(fp, "/DOPLER/    %8.1f", 1000*BU[k]);
	    
	    y[0] =  GetXS(lib, XS_DIFF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_DIFF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[1], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_DIFF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[2], 
			  VOI[0], RHO[0], EXP[k]);

	    p = Fit2(x, y);

	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_SIGA1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_SIGA1, VHI[0], BHI[0], BOR[0], CR[0], TFU[1], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_SIGA1, VHI[0], BHI[0], BOR[0], CR[0], TFU[2], 
			  VOI[0], RHO[0], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E", p[1], p[0]);
	    
	    y[0] =  GetXS(lib, XS_NUSF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_NUSF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[1], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_NUSF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[2], 
			  VOI[0], RHO[0], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E\n         %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E\n         %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_SIG12, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_SIG12, VHI[0], BHI[0], BOR[0], CR[0], TFU[1], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_SIG12, VHI[0], BHI[0], BOR[0], CR[0], TFU[2], 
			  VOI[0], RHO[0], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_DIFF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_DIFF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[1], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_DIFF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[2], 
			  VOI[0], RHO[0], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_SIGA2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_SIGA2, VHI[0], BHI[0], BOR[0], CR[0], TFU[1], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_SIGA2, VHI[0], BHI[0], BOR[0], CR[0], TFU[2], 
			  VOI[0], RHO[0], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E\n         %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E\n         %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_NUSF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_NUSF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[1], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_NUSF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[2], 
			  VOI[0], RHO[0], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E %11.4E\n", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E\n", p[1], p[0]);
	  }

      /* Get density vector */

      x[0] = 0.0;
      x[1] = RHO[1] - RHO[0];
      x[2] = RHO[2] - RHO[0];

      /* Loop over burnups */

      if (ntm == 3)
	for (k = 0; k < nbu; k++)
	  {
	    fprintf(fp, "/MODER /1   %8.1f", 1000*BU[k]);
	    
	    y[0] =  GetXS(lib, XS_DIFF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_DIFF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[1], EXP[k]);
	    y[2] =  GetXS(lib, XS_DIFF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[2], EXP[k]);
	    
	    y[0] = 1.0/y[0];
	    y[1] = 1.0/y[1];
	    y[2] = 1.0/y[2];

	    p = Fit2(x, y);


	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_SIGA1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_SIGA1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[1], EXP[k]);
	    y[2] =  GetXS(lib, XS_SIGA1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[2], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E", p[1], p[0]);
	    
	    y[0] =  GetXS(lib, XS_NUSF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_NUSF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[1], EXP[k]);
	    y[2] =  GetXS(lib, XS_NUSF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[2], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E\n         %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E\n         %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_SIG12, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_SIG12, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[1], EXP[k]);
	    y[2] =  GetXS(lib, XS_SIG12, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[2], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_DIFF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_DIFF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[1], EXP[k]);
	    y[2] =  GetXS(lib, XS_DIFF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[2], EXP[k]);

	    y[0] = 1.0/y[0];
	    y[1] = 1.0/y[1];
	    y[2] = 1.0/y[2];
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_SIGA2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_SIGA2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[1], EXP[k]);
	    y[2] =  GetXS(lib, XS_SIGA2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[2], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E\n         %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E\n         %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_NUSF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_NUSF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[1], EXP[k]);
	    y[2] =  GetXS(lib, XS_NUSF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[2], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HFP)
	      fprintf(fp, " %11.4E %11.4E\n", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E\n", p[1], p[0]);
	  }

      /* Boron fitting is done for concentration * density */
      
      x[0] = 0.0;
      x[1] = RHO[0]*(BOR[1] - BOR[0]);
      x[2] = RHO[0]*(BOR[2] - BOR[0]);

      /* Loop over burnups */

      if (nbo == 3)
	for (k = 0; k < nbu; k++)
	  {
	    fprintf(fp, "/BORON /    %8.1f", 1000*BU[k]);
	    
	    y[0] =  GetXS(lib, XS_DIFF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_DIFF1, VHI[0], BHI[0], BOR[1], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_DIFF1, VHI[0], BHI[0], BOR[2], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HZP)
	      fprintf(fp, " %11.4E %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_SIGA1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_SIGA1, VHI[0], BHI[0], BOR[1], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_SIGA1, VHI[0], BHI[0], BOR[2], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HZP)
	      fprintf(fp, " %11.4E %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E", p[1], p[0]);
	    
	    y[0] =  GetXS(lib, XS_NUSF1, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_NUSF1, VHI[0], BHI[0], BOR[1], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_NUSF1, VHI[0], BHI[0], BOR[2], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HZP)
	      fprintf(fp, " %11.4E\n         %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E\n         %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_SIG12, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_SIG12, VHI[0], BHI[0], BOR[1], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_SIG12, VHI[0], BHI[0], BOR[2], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HZP)
	      fprintf(fp, " %11.4E %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_DIFF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_DIFF2, VHI[0], BHI[0], BOR[1], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_DIFF2, VHI[0], BHI[0], BOR[2], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HZP)
	      fprintf(fp, " %11.4E %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_SIGA2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_SIGA2, VHI[0], BHI[0], BOR[1], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_SIGA2, VHI[0], BHI[0], BOR[2], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HZP)
	      fprintf(fp, " %11.4E\n         %11.4E", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E\n         %11.4E", p[1], p[0]);

	    y[0] =  GetXS(lib, XS_NUSF2, VHI[0], BHI[0], BOR[0], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[1] =  GetXS(lib, XS_NUSF2, VHI[0], BHI[0], BOR[1], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    y[2] =  GetXS(lib, XS_NUSF2, VHI[0], BHI[0], BOR[2], CR[0], TFU[0], 
			  VOI[0], RHO[0], EXP[k]);
	    
	    p = Fit2(x, y);

	    if (mode < MODE_HZP)
	      fprintf(fp, " %11.4E %11.4E\n", 0.0, 0.0);
	    else
	      fprintf(fp, " %11.4E %11.4E\n", p[1], p[0]);
	  }
      
      /***********************************************************************/

      /* Close file */

      fclose(fp);

      /* Next library */

      lib = NextItem(lib);
    }

  /* Newline */

  printf("\n");
}

/*****************************************************************************/
