/*****************************************************************************/
/*                                                                           */
/* AresFit : processares.c                                                   */
/*                                                                           */
/* Created:       2014/08/25 (JLe)                                           */
/* Last modified: 2014/09/21 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: Main data processing routine for ARES                        */ 
/*                                                                           */
/* Comments: - Tässä oletetaan nyt että moderaattorin lämpötila- ja boori-   */
/*             historioita on aina 3 kumpaakin.                              */
/*                                                                           */
/*****************************************************************************/

#include "header.h"
#include "locations.h"

#define FUNCTION_NAME "ProcessARES:"

/* Coefficient indexes */

#define iA 1
#define iB 2
#define iC 3
#define iD 4
#define iE 5
#define iF 6
#define iG 7
#define iH 8
#define iI 9
#define iJ 10
#define iK 11
#define iL 12
#define iM 13
#define iN 14
#define iO 15
#define iP 16
#define iQ 17
#define iR 18
#define iS 19
#define iT 20
#define iU 21
#define iV 22
#define iW 23

/* Matriisin indeksointi: */

/*
1 - CR
2 - VHI
3 - EXP
4 - DEN
5 - TFU
6 - TMO
7 - CRH (ei käytetä)
8 - BOR
9 - BHI
*/

/*****************************************************************************/

void ProcessARES()
{
  long lib, xsp, his, para, ncr, nvh, nbu, ntf, nvo, ntm, nbo, nbh;
  long i, j, k, l, ic, iv, ib, n, m, N, mode;
  double coe[24];
  double VHI[3], BHI[3], BOR[3], CR[3], TFU[3], VOI[3], TMO[3], EXP[100];
  FILE *fp;
  complex **A, X[23], Y[23];

  /* Allocate memory for matrix */

  A = (complex **)Mem(MEM_ALLOC, 23, sizeof(complex)); 
  for (i = 0; i < 23; i++)
    A[i] = (complex *)Mem(MEM_ALLOC, 23, sizeof(complex)); 

  /* Get mode */

  mode = (long)RDB[DATA_MODE];

  /* Loop over libraries */

  lib = (long)RDB[DATA_PTR_LIB0];
  while (lib > VALID_PTR)
    {
      /* Print */
      
      printf("Processing library '%s'...\n", GetText(lib + LIB_PTR_NAME));

      /* Open file */

      fp = OpenFile(GetText(lib + LIB_PTR_NAME), "w");

      /* Print stuff at beginning of file */

      fprintf(fp, "CROSS SECTION DATA FROM SERPENT RUN:\n\n");
      fprintf(fp, "S = S_base + (a*x1+b)*x1 + (c*x2+d)*x2 + (e*x3+f)*x3 + g*x1*x2 + (h*x3+i)*x1*x3 + j*x2*x3 + k*x4 + (l*x5+m)*x5 + (n*x6+o)*x6 + (p*x5^2*x3^2+q*x5^2*x3+v*x5*x3^2+w*x5*x3) + (r*x5+s)*x5*x2 + (t*x5+u)*x5*x1\n\n");
      fprintf(fp, "x1 = relative coolant density - nominal value\n");
      fprintf(fp, "x2 = sqrt(TFU) - nominal value\n");
      fprintf(fp, "x3 = moderator temperature - nominal value\n");
      fprintf(fp, "x4 = control rod history\n");
      fprintf(fp, "x5 = boron concentration\n");
      fprintf(fp, "x6 = boron history\n\n");
      fprintf(fp, "Nominal values:\n");
      fprintf(fp, "         DEN          TFU          TMO\n");
      
      /* Print nominal values */
      
      fprintf(fp, " %11.5E %11.5E %11.5E\n", 1.0, 
 	      sqrt(RDB[lib + LIB_NOM_TFU]), RDB[lib + LIB_NOM_TMO]);
      
      /* Loop over parameters */

      for (n = 0; n < 40; n++)
	{
	  /*******************************************************************/
	      
	  /***** Select parameter ********************************************/

	  if (n == 0)
	    {
	      para = XS_SIGA1;
	      fprintf(fp, "   SIGA1\n");
	    }
	  else if (n == 1)
	    {
	      para = XS_SIGA2;
	      fprintf(fp, "   SIGA2\n");
	    }
	  else if (n == 2)
	    {
	      para = XS_NUSF1;
	      fprintf(fp, "   NUSF1\n");
	    }
	  else if (n == 3)
	    {
	      para = XS_NUSF2;
	      fprintf(fp, "   NUSF2\n");
	    }
	  else if (n == 4)
	    {
	      para = XS_DIFF1;
	      fprintf(fp, "   DIFF1\n");
	    }
	  else if (n == 5)
	    {
	      para = XS_DIFF2;
	      fprintf(fp, "   DIFF2\n");
	    }
	  else if (n == 6)
	    {
	      para = XS_SIG12;
	      fprintf(fp, "   SIG12\n");
	    }
	  else if (n == 7)
	    {
	      para = XS_EPSF1;
	      fprintf(fp, "   EPSF1\n");
	    }
	  else if (n == 8)
	    {
	      para = XS_EPSF2;
	      fprintf(fp, "   EPSF2\n");
	    }
	  else if (n == 9)
	    {
	      para = XS_SIGF1;
	      fprintf(fp, "   SIGF1\n");
	    }
	  else if (n == 10)
	    {
	      para = XS_SIGF2;
	      fprintf(fp, "   SIGF2\n");
	    }
	  else if (n == 11)
	    {
	      para = XS_SDF1W;
	      fprintf(fp, "   SDF1W\n");
	    }
	  else if (n == 12)
	    {
	      para = XS_SDF2W;
	      fprintf(fp, "   SDF2W\n");
	    }
	  else if (n == 13)
	    {
	      para = XS_SDF1S;
	      fprintf(fp, "   SDF1S\n");
	    }
	  else if (n == 14)
	    {
	      para = XS_SDF2S;
	      fprintf(fp, "   SDF2S\n");
	    }
	  else if (n == 15)
	    {
	      para = XS_SDF1E;
	      fprintf(fp, "   SDF1E\n");
	    }
	  else if (n == 16)
	    {
	      para = XS_SDF2E;
	      fprintf(fp, "   SDF2E\n");
	    }
	  else if (n == 17)
	    {
	      para = XS_SDF1N;
	      fprintf(fp, "   SDF1N\n");
	    }
	  else if (n == 18)
	    {
	      para = XS_SDF2N;
	      fprintf(fp, "   SDF2N\n");
	    }
	  else if (n == 19)
	    {
	      para = XS_CD1SW;
	      fprintf(fp, "   CD1SW\n");
	    }
	  else if (n == 20)
	    {
	      para = XS_CD2SW;
	      fprintf(fp, "   CD2SW\n");
	    }
	  else if (n == 21)
	    {
	      para = XS_CD1SE;
	      fprintf(fp, "   CD1SE\n");
	    }
	  else if (n == 22)
	    {
	      para = XS_CD2SE;
	      fprintf(fp, "   CD2SE\n");
	    }
	  else if (n == 23)
	    {
	      para = XS_CD1NE;
	      fprintf(fp, "   CD1NE\n");
	    }
	  else if (n == 24)
	    {
	      para = XS_CD2NE;
	      fprintf(fp, "   CD2NE\n");
	    }
	  else if (n == 25)
	    {
	      para = XS_CD1NW;
	      fprintf(fp, "   CD1NW\n");
	    }
	  else if (n == 26)
	    {
	      para = XS_CD2NW;
	      fprintf(fp, "   CD2NW\n");
	    }
	  else if (n == 27)
	    {
	      para = XS_SIGI1;
	      fprintf(fp, "   SIGI1\n");
	    }
	  else if (n == 28)
	    {
	      para = XS_SIGI2;
	      fprintf(fp, "   SIGI2\n");
	    }
	  else if (n == 29)
	    {
	      para = XS_SIGI3;
	      fprintf(fp, "   SIGI3\n");
	    }
	  else if (n == 30)
	    {
	      para = XS_SIGX1;
	      fprintf(fp, "   SIGX1\n");
	    }
	  else if (n == 31)
	    {
	      para = XS_SIGX2;
	      fprintf(fp, "   SIGX2\n");
	    }
	  else if (n == 32)
	    {
	      para = XS_SIGX3;
	      fprintf(fp, "   SIGX3\n");
	    }
	  else if (n == 33)
	    {
	      para = XS_SIGP1;
	      fprintf(fp, "   SIGP1\n");
	    }
	  else if (n == 34)
	    {
	      para = XS_SIGP2;
	      fprintf(fp, "   SIGP2\n");
	    }
	  else if (n == 35)
	    {
	      para = XS_SIGP3;
	      fprintf(fp, "   SIGP3\n");
	    }
	  else if (n == 36)
	    {
	      para = XS_SIGS1;
	      fprintf(fp, "   SIGS1\n");
	    }
	  else if (n == 37)
	    {
	      para = XS_SIGS2;
	      fprintf(fp, "   SIGS2\n");
	    }
	  else if (n == 38)
	    {
	      para = XS_SIGS3;
	      fprintf(fp, "   SIGS3\n");
	    }
	  else if (n == 39)
	    {
	      para = XS_BETA ;
	      fprintf(fp, "    BETA\n"); /* HUOM! merkkijonon oikea kohta */
	    }
	  else if (n == 40)
	    {
	      para = XS_BETA1;
	      fprintf(fp, "   BETA1\n");
	    }
	  else if (n == 41)
	    {
	      para = XS_BETA2;
	      fprintf(fp, "   BETA2\n");
	    }
	  else if (n == 42)
	    {
	      para = XS_BETA3;
	      fprintf(fp, "   BETA3\n");
	    }
	  else if (n == 43)
	    {
	      para = XS_BETA4;
	      fprintf(fp, "   BETA4\n");
	    }
	  else if (n == 44)
	    {
	      para = XS_BETA5;
	      fprintf(fp, "   BETA5\n");
	    }
	  else if (n == 45)
	    {
	      para = XS_BETA6;
	      fprintf(fp, "   BETA6\n");
	    }
	  else if (n == 46)
	    {
	      para = XS_INVV1;
	      fprintf(fp, "   INVV1\n");
	    }
	  else if (n == 47)
	    {
	      para = XS_INVV2;
	      fprintf(fp, "   INVV2\n");
	    }
	  else if (n == 48)
	    {
	      para = XS_LBDA1;
	      fprintf(fp, "   LBDA1\n");
	    }
	  else if (n == 49)
	    {
	      para = XS_LBDA2;
	      fprintf(fp, "   LBDA2\n");
	    }
	  else if (n == 50)
	    {
	      para = XS_LBDA3;
	      fprintf(fp, "   LBDA3\n");
	    }
	  else if (n == 51)
	    {
	      para = XS_LBDA4;
	      fprintf(fp, "   LBDA4\n");
	    }
	  else if (n == 52)
	    {
	      para = XS_LBDA5;
	      fprintf(fp, "   LBDA5\n");
	    }
	  else if (n == 53)
	    {
	      para = XS_LBDA6;
	      fprintf(fp, "   LBDA6\n");
	    }
	  else
	    Die(FUNCTION_NAME, "Overflow");

	  /*******************************************************************/

	  /***** Collect variables into vectors ******************************/

	  /* Reset maximum number of branches */

	  ncr = 0; 
	  nvh = 0; 
	  nbu = 0;
	  ntf = 0;
	  nvo = 0;
	  ntm = 0;
	  nbo = 0;
	  nbh = 0;
	  
	  /* Reset data */
	  
	  for (i = 0; i < 3; i++)
	    VHI[i] = 0.0;
	  
	  for (i = 0; i < 3; i++)
	    BHI[i] = 0.0;
	  
	  for (i = 0; i < 3; i++)
	    BOR[i] = 0.0;
	  
	  for (i = 0; i < 3; i++)
	    TFU[i] = 0.0;
	  
	  for (i = 0; i < 3; i++)
	    VOI[i] = 0.0;
	  
	  for (i = 0; i < 3; i++)
	    TMO[i] = 0.0;
	  
	  for (i = 0; i < 3; i++)
	    EXP[i] = 0.0;
	  
	  for (i = 0; i < 3; i++)
	    CR[i] = 0.0;

	  /* Loop over histories */

	  his = (long)RDB[lib + LIB_PTR_HIS];
	  while (his > VALID_PTR)
	    {
	      /* Loop over data */
	      
	      xsp = (long)RDB[his + HIS_PTR_XS];
	      while (xsp > VALID_PTR)
		{
		  /* Get parameter values */

		  i = (long)RDB[xsp + XS_VHI_IDX];
		  VHI[i] = RDB[xsp + XS_VHI];
		  
		  if (i + 1 > nvh)
		    nvh = i + 1;

		  i = (long)RDB[xsp + XS_BHI_IDX];
		  BHI[i] = RDB[xsp + XS_BHI];

		  if (i + 1 > nbh)
		    nbh = i + 1;

		  i = (long)RDB[xsp + XS_BOR_IDX];
		  BOR[i] = RDB[xsp + XS_BOR];

		  if (i + 1 > nbo)
		    nbo = i + 1;

		  i = (long)RDB[xsp + XS_TFU_IDX];
		  TFU[i] = RDB[xsp + XS_TFU];

		  if (i + 1 > ntf)
		    ntf = i + 1;

		  i = (long)RDB[xsp + XS_VOI_IDX];
		  VOI[i] = RDB[xsp + XS_VOI];

		  if (i + 1 > nvo)
		    nvo = i + 1;
		  
		  i = (long)RDB[xsp + XS_TMO_IDX];
		  TMO[i] = RDB[xsp + XS_TMO];
		  
		  if (i + 1 > ntm)
		    ntm = i + 1;

		  i = (long)RDB[xsp + XS_EXP_IDX];

		  if (i > 99)
		    Die(FUNCTION_NAME, "Increase vector size");
		  else
		    EXP[i] = RDB[xsp + XS_EXP];
		  
		  if (i + 1 > nbu)
		    nbu = i + 1;

		  i = (long)RDB[xsp + XS_CR_IDX];
		  CR[i] = RDB[xsp + XS_CR];

		  if (i + 1 > ncr)
		    ncr = i + 1;

		  /* Next */
		  
		  xsp = NextItem(xsp);
		}
	        
	      /* Next history */
	  
	      his = NextItem(his);
	    }

	  /* Check */
	  
	  if (mode > MODE_HZP)
	    {
	      if (ntf < 3)
		Die(FUNCTION_NAME, 
		    "Insufficient number of fuel temperature branches");
	  
	      if (nvo < 3)
		Die(FUNCTION_NAME, 
		    "Insufficient number of coolant void branches");
	      
	      if (ntm < 3)
		Die(FUNCTION_NAME, 
		    "Insufficient number of moderator temperature branches");
	    }
	  
	  if (mode > MODE_BAS)
	    if (nbo < 3)
	      Die(FUNCTION_NAME, 
		  "Insufficient number of boron concentration branches");

	  /* Print (debug) */
	
	  if (1 == 2)
	    {
	      for (i = 0; i < nvh; i++)
		printf("VHI = %E\n", VHI[i]);
	      
	      for (i = 0; i < 3; i++)
		printf("BHI = %E\n", BHI[i]);
	      
	      for (i = 0; i < 3; i++)
		printf("BOR = %E\n", BOR[i]);
	      
	      for (i = 0; i < 3; i++)
		printf("TFU = %E\n", TFU[i]);
	      
	      for (i = 0; i < 3; i++)
		printf("VOI = %E\n", VOI[i]);
	      
	      for (i = 0; i < 3; i++)
		printf("TMO = %E\n", TMO[i]);
	      
	      for (i = 0; i < nbu; i++)
		printf("EXP = %E\n", EXP[i]);
	      
	      for (i = 0; i < ncr; i++)
		printf("CR = %E\n", CR[i]);
	    }

	  /* Check nominal values */

	  if (mode > MODE_HZP)
	    {
	      if (TFU[0] != RDB[lib + LIB_NOM_TFU])
		Die(NULL, "Mismatch in TFU nominal value");
	      
	      if (VOI[0] != RDB[lib + LIB_NOM_VOI])
		Die(NULL, "Mismatch in VOI nominal value");
	      
	      if (TMO[0] != RDB[lib + LIB_NOM_TMO])
		Die(NULL, "Mismatch in TMO nominal value");
	    }

	  if (mode > MODE_BAS)
	    if (BOR[0] != 0.0)
	      Die(NULL, "Mismatch in BOR nominal value");

	  /*******************************************************************/

	  /***** Fit & print *************************************************/

	  /* Set number of burnup points if not in depletion mode */

	  if (mode != MODE_DEP)
	    nbu = 3;

	  /* Print header */

	  fprintf(fp, "CR   VHI   EXP       S_base            a            b            c            d            e            f            g            h            i            j            k            l            m            n            o            p            q            r            s            t            u            v            w\n");

	  /* Loop over control rod branches, void / moderator temperature */
	  /* history and burnup */

	  for (i = 0; i < 3; i++)
	  for (j = 0; j < 3; j++)
	  for (k = 0; k < nbu; k++)
	    {

	      /* Reset matrix and vectors */
	      
	      for (l = 0; l < 23; l++)
		for (m = 0; m < 23; m++)
		  {
		    A[l][m].re = 0.0;
		    A[l][m].im = 0.0;
		  }

	      for (l = 0; l < 23; l++)
		{
		  X[l].re = 0.0;
		  X[l].im = 0.0;
		  Y[l].re = 0.0;
		  Y[l].im = 0.0;
		}

	      /* Override control rod void and burnup indexes */

	      if (i < ncr)
		ic = i;
	      else
		ic = ncr - 1;

	      if (mode != MODE_DEP)
		iv = 0;
	      else if (j < nvh)
		iv = j;
	      else
		iv = nvh - 1;

	      if (mode != MODE_DEP)
		ib = 0;
	      else
		ib = k;
	      
	      /* Print control rod branch */ 

	      fprintf(fp, "%2ld ", i);

	      /* Print void / moderator temperature history */

	      fprintf(fp, "%5.1f ", VHI[iv]);

	      /* Print burnup */

	      if (mode != MODE_DEP)
		{
		  /* No actual burnup points, print identical entries */
		  /* with different burnup. */

		  if (k == 0)
		    fprintf(fp, "  0.0");
		  else if (k == 1)
		    fprintf(fp, "  0.1");
		  else
		    fprintf(fp, "  0.5");
		}
	      else
		fprintf(fp, "%5.1f", EXP[ib]);

	      /* NOTE: Tää pakottaa nollapalaman vakiot samoiksi kaikilla */
	      /* historioilla. Jostain syystä ARES kaatuu jos näin ei ole */
	      /* (kysy Rikulta!!!) */
	      
	      if (ib == 0)
		iv = 0;

	      /* Reset coefficient vector */
		  
	      for (m = 0; m < 24; m++)
		coe[m] = 0.0;

	      /***************************************************************/

	      /***** Add points to matrix ************************************/

	      /* Reset count */

	      N = 0;

	      /* Nominal state */

	      N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[0], CR[ic],
			   TFU[0], VOI[0], TMO[0], EXP[ib], NO);

	      /* Check if state coefficients are included */
	      
	      if (mode > MODE_HZP)
		{
		  /* Coolant void branches */

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[0], 
			       CR[ic], TFU[0], VOI[1], TMO[0], EXP[ib], NO);
	      
		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[0], 
			       CR[ic], TFU[0], VOI[2], TMO[0], EXP[ib], NO);

		  /* Fuel temperature branches */
		  
		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[0], 
			       CR[ic], TFU[1], VOI[0], TMO[0], EXP[ib], NO);
		  
		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[0], 
			       CR[ic], TFU[2], VOI[0], TMO[0], EXP[ib], NO);

		  /* Moderator temperature branches */

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[0], 
			       CR[ic], TFU[0], VOI[0], TMO[1], EXP[ib], NO);

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[0], 
			       CR[ic], TFU[0], VOI[0], TMO[2], EXP[ib], NO);
	      
		  /* Void / fuel temperature cross term */
	      
		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[0], 
			       CR[ic], TFU[2], VOI[2], TMO[0], EXP[ib], NO);

		  /* Void / moderator temperature cross terms */

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[0], 
			       CR[ic], TFU[0], VOI[1], TMO[1], EXP[ib], NO);

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[0], 
			       CR[ic], TFU[0], VOI[2], TMO[2], EXP[ib], NO);

		  /* Moderator temperature / fuel temperature cross term */

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[0], 
			       CR[ic], TFU[2], VOI[0], TMO[2], EXP[ib], NO);
		}

	      /* Check if control coefficients are included */

	      if (mode > MODE_BAS)
		{
		  /* Boron concentration branches */
		  
		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[1], 
			       CR[ic], TFU[0], VOI[0], TMO[0], EXP[ib], NO);

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[2], 
			       CR[ic], TFU[0],  VOI[0], TMO[0], EXP[ib], NO);
		}

	      /* Check if history coefficients are included */

	      if (mode > MODE_HFP)
		{
		  /* Boron history branches */
		  
		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[1], BOR[0], 
			       CR[ic], TFU[0], VOI[0], TMO[0], EXP[ib], NO);
		  
		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[2], BOR[0], 
			       CR[ic], TFU[0],  VOI[0], TMO[0], EXP[ib], NO);
		}

	      /* Check if state coefficients are included */
	      
	      if (mode > MODE_HZP)
		{
		  /* Moderator temperature / boron concentration cross terms */

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[1], 
			       CR[ic], TFU[0], VOI[0], TMO[1], EXP[ib], NO);

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[2], 
			       CR[ic], TFU[0], VOI[0], TMO[1], EXP[ib], NO);

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[1], 
			       CR[ic], TFU[0], VOI[0], TMO[2], EXP[ib], NO);

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[2], 
			       CR[ic], TFU[0], VOI[0], TMO[2], EXP[ib], NO);

		  /* Fuel temperature / boron concentration cross terms */

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[1], 
			       CR[ic], TFU[1], VOI[0], TMO[0], EXP[ib], NO);

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[2], 
			       CR[ic], TFU[1], VOI[0], TMO[0], EXP[ib], NO);
	      
		  /* Void / boron concentration cross terms */
	      
		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[1], 
			       CR[ic], TFU[0], VOI[1], TMO[0], EXP[ib], NO);

		  N = AddPoint(N, lib, para, A, Y, VHI[iv], BHI[0], BOR[2], 
			       CR[ic], TFU[0], VOI[2], TMO[0], EXP[ib], NO);
		}

	      /* Solve matrix equation */

	      LUdecomposition(N, A, Y, X);

	      /* Put values */

	      m = 0;
	      coe[0] = X[m++].re;

	      if (mode > MODE_HZP)
		{
		  coe[iA] = X[m++].re;
		  coe[iB] = X[m++].re;
		  coe[iC] = X[m++].re;
		  coe[iD] = X[m++].re;
		  coe[iE] = X[m++].re;
		  coe[iF] = X[m++].re;
		  coe[iG] = X[m++].re;
		  
		  coe[iH] = X[m++].re;
		  coe[iI] = X[m++].re;
		  
		  coe[iJ] = X[m++].re;
		}

	      /*
	      coe[iK] = X[m++].re;
	      */
	      
	      if (mode > MODE_BAS)
		{
		  coe[iL] = X[m++].re;
		  coe[iM] = X[m++].re;
		}
	      if (mode > MODE_HFP)
		{
		  coe[iN] = X[m++].re;
		  coe[iO] = X[m++].re;
		}
	      if (mode > MODE_HZP)
		{
		  coe[iP] = X[m++].re;
		  coe[iQ] = X[m++].re;
		  coe[iV] = X[m++].re;
		  coe[iW] = X[m++].re;
		  
		  coe[iR] = X[m++].re;
		  coe[iS] = X[m++].re;
		  coe[iT] = X[m++].re;
		  coe[iU] = X[m++].re;
		}

	      /* Test data */

	      if ((mode > MODE_BAS) && (mode < MODE_DEP))
		TestAres(lib, para, VHI[iv], CR[ic], EXP[ib], BHI[0], coe);
	      else if (mode == MODE_DEP)
		for (m = 0; m < 3; m++)
		  TestAres(lib, para, VHI[iv], CR[ic], EXP[ib], BHI[m], coe);

	      /***************************************************************/

	      /***** Print vector ********************************************/

	      /* Check base value */
	      
	      if (coe[0] < 0.0)
		Die(FUNCTION_NAME, "Invalid value %E (%ld)", coe[0], n);

	      /* Print vector */
	      
	      for (m = 0; m < 24; m++)
		fprintf(fp, " %12.5e", coe[m]);

	      /* Newline */
	      
	      fprintf(fp, "\n");
	      
	      /***************************************************************/

	    }
	
	  /*******************************************************************/
	}
    
      /***********************************************************************/

      /***** Count number of files used **************************************/

      /* Reset counters */

      n = 0;
      m = 0;
      i = 0;
      j = 0;

      /* Loop over histories */

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  /* Loop over data */

	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Add to total (if-statement avoids counting other void */
	      /* histories for zero burnup which were excluded earlir) */
	      
	      if (((long)RDB[xsp + XS_EXP_IDX] > 0) ||
		  ((long)RDB[xsp + XS_VHI_IDX] == 0))
		{
		  n++;
		  i++;
		}

	      /* Add to used */

	      if ((long)RDB[xsp + XS_USED] > 0)
		m++;
	      else if (((long)RDB[xsp + XS_EXP_IDX] > 0) ||
		       ((long)RDB[xsp + XS_VHI_IDX] == 0))
		{
		  printf("\nNot used :  VHI = %1.2f\n", RDB[xsp + XS_VHI]);
		  printf("            BHI = %1.2f\n", RDB[xsp + XS_BHI]);
		  printf("            BOR = %1.2f\n", RDB[xsp + XS_BOR]);
		  printf("            CR  = %1.0f\n", RDB[xsp + XS_CR]);
		  printf("            TFU = %1.2f\n", RDB[xsp + XS_TFU]);
		  printf("            VOI = %1.2f\n", RDB[xsp + XS_VOI]);
		  printf("            TMO = %1.2f\n", RDB[xsp + XS_TMO]);
		  printf("            EXP = %1.2f\n", RDB[xsp + XS_EXP]);
		}

	      /* Add to tested */

	      if ((long)RDB[xsp + XS_TESTED] > 0)
		j++;
	      else if (((long)RDB[xsp + XS_EXP_IDX] > 0) ||
		       ((long)RDB[xsp + XS_VHI_IDX] == 0))
		{
		  printf("\nNot tested :  VHI = %1.2f\n", RDB[xsp + XS_VHI]);
		  printf("              BHI = %1.2f\n", RDB[xsp + XS_BHI]);
		  printf("              BOR = %1.2f\n", RDB[xsp + XS_BOR]);
		  printf("              CR  = %1.0f\n", RDB[xsp + XS_CR]);
		  printf("              TFU = %1.2f\n", RDB[xsp + XS_TFU]);
		  printf("              VOI = %1.2f\n", RDB[xsp + XS_VOI]);
		  printf("              TMO = %1.2f\n", RDB[xsp + XS_TMO]);
		  printf("              EXP = %1.2f\n", RDB[xsp + XS_EXP]);
		}
	      
	      /* Next */

	      xsp = NextItem(xsp);
	    }

	  /* Next history */

	  his = NextItem(his);
	}

      printf("\n %4ld / %4ld files used.", m, n);
      printf("\n %4ld / %4ld files tested.\n\n", j, i);

      /***********************************************************************/

      /* Close file */

      fclose(fp);

      /* Next library */

      lib = NextItem(lib);
    }
  
  /* Free allocated memory */

  for (i = 0; i < 23; i++)
    Mem(MEM_FREE, A[i]);
  
  Mem(MEM_FREE, A);
}

/*****************************************************************************/

/* Tällä saatiin lisättyä ne puuttuvat */




#ifdef mmmm
      n = 0;

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  /* Loop over data */

	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Add to used */

	      if ((long)RDB[xsp + XS_USED] > 0)
		{
		  printf("\n%% --- BOR %1.0f TFU %1.0f TMO %1.0f VOI %1.2f:\n\n", RDB[xsp + XS_BOR], RDB[xsp + XS_TFU], RDB[xsp + XS_TMO], RDB[xsp + XS_VOI]);
		  
		  printf("branch BRA%ld\n\n", n++);

		  if ((long)RDB[xsp + XS_BOR_IDX] == 0)
		    {
		      printf("repm cool cool_0B\n");
		      printf("repm coolc coolc_0B\n");
		      printf("var BOR 0.0\n");
		    }
		  else if ((long)RDB[xsp + XS_BOR_IDX] == 1)
		    {
		      printf("repm cool cool_midB\n");
		      printf("repm coolc coolc_midB\n");
		      printf("var BOR 750.0\n");
		    }
		  else if ((long)RDB[xsp + XS_BOR_IDX] == 2)
		    {
		      printf("repm cool cool_hiB\n");
		      printf("repm coolc coolc_hiB\n");
		      printf("var BOR 1500.0\n");
		    }

		  if ((long)RDB[xsp + XS_TFU_IDX] == 0)
		    {
		      printf("stp fuel16  sum  900\n");
		      printf("stp fuel24  sum  900\n");
		      printf("stp fuel31  sum  900\n");
		      printf("var TFU 900.0\n");
		    }
		  else if ((long)RDB[xsp + XS_TFU_IDX] == 1)
		    {
		      printf("stp fuel16  sum  600\n");
		      printf("stp fuel24  sum  600\n");
		      printf("stp fuel31  sum  600\n");
		      printf("var TFU 600.0\n");
		    }
		  else if ((long)RDB[xsp + XS_TFU_IDX] == 2)
		    {
		      printf("stp fuel16  sum  1200\n");
		      printf("stp fuel24  sum  1200\n");
		      printf("stp fuel31  sum  1200\n");
		      printf("var TFU 1200.0\n");
		    }

		  if (((long)RDB[xsp + XS_TMO_IDX] == 0) &&
		      ((long)RDB[xsp + XS_VOI_IDX] == 0))
		    {
		      printf("stp water -0.76971 550\n");
		      printf("var TMO 550\n");
		      printf("var VOI	0.00\n");
		    }
		  else if (((long)RDB[xsp + XS_TMO_IDX] == 1) &&
			   ((long)RDB[xsp + XS_VOI_IDX] == 0))
		    {
		      printf("stp water -0.84278 500\n");
		      printf("var TMO 500\n");
		      printf("var VOI	0.00\n");
		    }
		  else if (((long)RDB[xsp + XS_TMO_IDX] == 2) &&
			   ((long)RDB[xsp + XS_VOI_IDX] == 0))
		    {
		      printf("stp water -0.66114 600\n");
		      printf("var TMO 600\n");
		      printf("var VOI	0.00\n");
		    }
		  else if (((long)RDB[xsp + XS_TMO_IDX] == 0) &&
			   ((long)RDB[xsp + XS_VOI_IDX] == 1))
		    {
		      printf("stp water -0.73122 550\n");
		      printf("var TMO 550\n");
		      printf("var VOI	0.05\n");
		    }
		  else if (((long)RDB[xsp + XS_TMO_IDX] == 0) &&
			   ((long)RDB[xsp + XS_VOI_IDX] == 2))
		    {
		      printf("stp water -0.69274 550\n");
		      printf("var TMO 550\n");
		      printf("var VOI	0.10\n");
		    }
		  else if (((long)RDB[xsp + XS_TMO_IDX] == 1) &&
			   ((long)RDB[xsp + XS_VOI_IDX] == 1))
		    {
		      printf("stp water -0.80064 500\n");
		      printf("var TMO 500\n");
		      printf("var VOI	0.05\n");
		    }
		  else if (((long)RDB[xsp + XS_TMO_IDX] == 2) &&
			   ((long)RDB[xsp + XS_VOI_IDX] == 2))
		    {
		      printf("stp water -0.59503 600\n");
		      printf("var TMO 600\n");
		      printf("var VOI	0.10\n");
		    }

		}

	      /* Next */

	      xsp = NextItem(xsp);
	    }

	  /* Next history */

	  his = NextItem(his);
	}

      #endif

