/*****************************************************************************/
/*                                                                           */
/* AresFit : processparcs.c                                                  */
/*                                                                           */
/* Created:       2015/03/10 (JLe)                                           */
/* Last modified: 2015/03/10 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: Main data processing routine for PARCS                       */ 
/*                                                                           */
/* Comments:                                                                 */
/*                                                                           */
/*****************************************************************************/

#include "header.h"
#include "locations.h"

#define FUNCTION_NAME "ProcessPARCS:"

/*****************************************************************************/

void ProcessPARCS()
{
  long lib, xsp, his, ncr, nvh, nbu, ntf, nvo, ntm, nbo, nbh;
  long i, mode, nmat, n, flg, fiss;
  double VHI[10], BHI[10], BOR[10], CR[10], TFU[10], VOI[10], TMO[10], EXP[100];
  FILE *fp;
  char tmpstr[MAX_STR], STA_VAR[10][MAX_STR];

  /* Get mode */
    
  mode = (long)RDB[DATA_MODE];

  /* Reset material index */

  nmat = 0;
  
  /* Loop over libraries */

  lib = (long)RDB[DATA_PTR_LIB0];
  while (lib > VALID_PTR)
    {
      /* Reset variable list */

      for (n = 0; n < 10; n++)
	STA_VAR[n][0] = '\0';
      
      /* Print */

      printf("Processing library '%s'...\n", GetText(lib + LIB_PTR_NAME));

      /* Open file for writing */

      sprintf(tmpstr, "%s", GetText(lib + LIB_PTR_NAME));
      fp = OpenFile(tmpstr, "w");

      /*******************************************************************/
      
      /***** Collect variables into vectors ******************************/
      
      /* Reset maximum number of branches */
      
      ncr = 0; 
      nvh = 0; 
      nbu = 0;
      ntf = 0;
      nvo = 0;
      ntm = 0;
      nbo = 0;
      nbh = 0;
      
      /* Reset data */
      
      for (i = 0; i < 10; i++)
	VHI[i] = 0.0;
      
      for (i = 0; i < 10; i++)
	BHI[i] = 0.0;
      
      for (i = 0; i < 10; i++)
	BOR[i] = 0.0;
	  
      for (i = 0; i < 10; i++)
	TFU[i] = 0.0;
	  
      for (i = 0; i < 10; i++)
	VOI[i] = 0.0;
	  
      for (i = 0; i < 10; i++)
	TMO[i] = 0.0;
	  
      for (i = 0; i < 10; i++)
	EXP[i] = 0.0;
	  
      for (i = 0; i < 10; i++)
	CR[i] = 0.0;

      /* Loop over histories */

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  /* Loop over data */
	      
	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Get parameter values */

	      i = (long)RDB[xsp + XS_VHI_IDX];
	      VHI[i] = RDB[xsp + XS_VHI];
		  
	      if (i + 1 > nvh)
		nvh = i + 1;

	      i = (long)RDB[xsp + XS_BHI_IDX];
	      BHI[i] = RDB[xsp + XS_BHI];

	      if (i + 1 > nbh)
		nbh = i + 1;

	      i = (long)RDB[xsp + XS_BOR_IDX];
	      BOR[i] = RDB[xsp + XS_BOR];

	      if (i + 1 > nbo)
		nbo = i + 1;

	      i = (long)RDB[xsp + XS_TFU_IDX];
	      TFU[i] = RDB[xsp + XS_TFU];

	      if (i + 1 > ntf)
		ntf = i + 1;

	      i = (long)RDB[xsp + XS_VOI_IDX];
	      VOI[i] = RDB[xsp + XS_VOI];

	      if (i + 1 > nvo)
		nvo = i + 1;
		  
	      i = (long)RDB[xsp + XS_TMO_IDX];
	      TMO[i] = RDB[xsp + XS_TMO];
		  
	      if (i + 1 > ntm)
		ntm = i + 1;

	      i = (long)RDB[xsp + XS_EXP_IDX];

	      if (i > 99)
		Die(FUNCTION_NAME, "Increase vector size");
	      else
		EXP[i] = RDB[xsp + XS_EXP];
		  
	      if (i + 1 > nbu)
		nbu = i + 1;

	      i = (long)RDB[xsp + XS_CR_IDX];
	      CR[i] = RDB[xsp + XS_CR];

	      if (i + 1 > ncr)
		ncr = i + 1;

	      /* Add to state variables */

	      i = 0;
	      while (STA_VAR[i][0] != '\0')
		{
		  /* Compare */

		  if (!strcmp(STA_VAR[i], GetText(xsp + XS_PARCS_STA_VAR)))
		    break;

		  /* Next */
		  
		  i++;
		}

	      if (STA_VAR[i][0] == '\0')
		strcpy(STA_VAR[i], GetText(xsp + XS_PARCS_STA_VAR));
	      
	      /* Next */
		  
	      xsp = NextItem(xsp);
	    }
	        
	  /* Next history */
	  
	  his = NextItem(his);
	}

      /* Print (debug) */
      
      if (1 == 2)
	{
	  for (i = 0; i < nvh; i++)
	    printf("VHI = %E\n", VHI[i]);
	  
	  for (i = 0; i < nbh; i++)
	    printf("BHI = %E\n", BHI[i]);
	  
	  for (i = 0; i < nbo; i++)
	    printf("BOR = %E\n", BOR[i]);
	  
	  for (i = 0; i < ntf; i++)
	    printf("TFU = %E\n", TFU[i]);
	  
	  for (i = 0; i < nvo; i++)
	    printf("VOI = %E\n", VOI[i]);
	  
	  for (i = 0; i < ntm; i++)
	    printf("TMO = %E\n", TMO[i]);

	  for (i = 0; i < nbu; i++)
	    printf("EXP = %E\n", EXP[i]);
	  
	  for (i = 0; i < ncr; i++)
	    printf("CR = %E\n", CR[i]);
	}

      /************************************************************************/

      /***** Override some flags and parameters *******************************/

      /* Set default number of decay heat groups */

      WDB[lib + LIB_NHG] = 6.0;

     /* Pointer to first history */

      his = (long)RDB[lib + LIB_PTR_HIS];
      CheckPointer(FUNCTION_NAME, "(his)", DATA_ARRAY, his);
      
      /* Pointer to first data entry */

      xsp = (long)RDB[his + HIS_PTR_XS];
      CheckPointer(FUNCTION_NAME, "(his)", DATA_ARRAY, his);

      /* Set fissile flag */

      fiss = YES;

      /* Check fission xs */

      if (RDB[xsp + XS_SIGF1] < ZERO)
	{
	  /* Reset number of decay heat and delayed neutron groups */

	  WDB[lib + LIB_NDG] = 0.0;
	  WDB[lib + LIB_NHG] = 0.0;

	  /* Reset some options */

	  WDB[lib + LIB_PARCS_FLAGS] = 
	    (double)((long)RDB[lib + LIB_PARCS_FLAGS] & ~PARCS_FLAG_LXES);

	  WDB[lib + LIB_PARCS_FLAGS] = 
	    (double)((long)RDB[lib + LIB_PARCS_FLAGS] & ~PARCS_FLAG_LCHI);

	  WDB[lib + LIB_PARCS_FLAGS] = 
	    (double)((long)RDB[lib + LIB_PARCS_FLAGS] & ~PARCS_FLAG_LCHD);

	  WDB[lib + LIB_PARCS_FLAGS] = 
	    (double)((long)RDB[lib + LIB_PARCS_FLAGS] & ~PARCS_FLAG_LYLD);

	  WDB[lib + LIB_PARCS_FLAGS] = 
	    (double)((long)RDB[lib + LIB_PARCS_FLAGS] & ~PARCS_FLAG_LBET);

	  WDB[lib + LIB_PARCS_FLAGS] = 
	    (double)((long)RDB[lib + LIB_PARCS_FLAGS] & ~PARCS_FLAG_LAMB);

	  WDB[lib + LIB_PARCS_FLAGS] = 
	    (double)((long)RDB[lib + LIB_PARCS_FLAGS] & ~PARCS_FLAG_LDEC);

	  /* Reset fissile flag */

	  fiss = YES;
	}

      /************************************************************************/

      /***** GLOBAL_V *********************************************************/

      /* Get flags */

      flg = (long)RDB[lib + LIB_PARCS_FLAGS];

      /* Print title */

      fprintf(fp, "GLOBAL_V ");

      /* NSET number of history cases in PMAXS. number of XS sets in XESC */

      fprintf(fp, " %2ld", nvh*nbh);

      /* NGROUP Number of energy groups */

      fprintf(fp, " %2ld", (long)RDB[lib + LIB_NEG]);

      /* MDLAY Maximum number of delay neutron groups */

      fprintf(fp, " %2ld", (long)RDB[lib + LIB_NDG]);

      /* MDCAY Maximum number of decay Heat groups */

      fprintf(fp, " %2ld", (long)RDB[lib + LIB_NHG]);

      /* MADF Maximum number of ADF in each group */

      if (flg & PARCS_FLAG_LADF)
	fprintf(fp, " %2ld", (long)RDB[lib + LIB_INPUT_NSDF]);
      else
	fprintf(fp, "  0");

      /* MCDF Maximum number of CDF in each group */

      if (flg & PARCS_FLAG_LCDF)
	fprintf(fp, " %2ld", (long)RDB[lib + LIB_INPUT_NCDF]);
      else
	fprintf(fp, "  0");

      /* MRODS Maximum number of rods in computed part of assembly */

      fprintf(fp, "  1");
      
      /* MCLOA Maximum number of rod columns in whole assembly */

      fprintf(fp, "  1 ");
      
      /* Print flags */

      for (n = 0; n < 15; n++)
	{
	  if (flg & (long)pow(2,n))
	    fprintf(fp, " T");
	  else
	    fprintf(fp, " F");
	}
      
      fprintf(fp, "\n");

      /* 5 Comment lines */

      fprintf(fp, 
	      "*********************************************************\n");
      fprintf(fp, "* Title : %-45s *\n",  GetText(lib + LIB_PTR_NAME));
      fprintf(fp, "* Date : %-46s *\n", TimeStamp());
      fprintf(fp, "* Created from Serpent 2 output using %6s %5s      *\n", 
	      CODE_NAME, CODE_VERSION);
      fprintf(fp, 
	      "*********************************************************\n");
      
      /*
      fprintf(fp, " Contents of T/H Invariant Variables (TIV) block and ");
      fprintf(fp, "Cross Sections (XS) block\n");
      fprintf(fp, "       TIV: inV/Bet/\n");
      fprintf(fp, "        XS: tr,ab,nf,kf/sct/ADF/\n");
      fprintf(fp, " 2  Group value of each variable are put together ");
      fprintf(fp, "in a line.\n");
      fprintf(fp, " Some variables (separated by ',') share a ");
      fprintf(fp, "line, '/' means change line\n");
      fprintf(fp, " Generated by SXSFit\n");
      */

      /************************************************************************/

      /***** STA_VAR **********************************************************/

      /* Count number of variables */

      for (n = 0; n < 10; n++)
	if (STA_VAR[n][0] == '\0')
	  break;

      /* Check first */

      if (strcasecmp(STA_VAR[0], "RE"))
	Die(NULL, "First STA_VAR must be RE");

      /* Print title */

      fprintf(fp, "STA_VAR  %3ld", n - 1);
      
      /* Print variables */

      for (n = 0; n < 10; n++)
	if (!strcmp(STA_VAR[n], "CR"))
	  fprintf(fp, " %2s", STA_VAR[n]);

      for (n = 0; n < 10; n++)
	if (!strcmp(STA_VAR[n], "PC"))
	  fprintf(fp, " %2s", STA_VAR[n]);

      for (n = 0; n < 10; n++)
	if (!strcmp(STA_VAR[n], "TF"))
	  fprintf(fp, " %2s", STA_VAR[n]);

      for (n = 0; n < 10; n++)
	if (!strcmp(STA_VAR[n], "TC"))
	  fprintf(fp, " %2s", STA_VAR[n]);
      
      fprintf(fp, "\n");

      /************************************************************************/

      /***** BRANCHES *********************************************************/

      /* Print title */

      fprintf(fp, "BRANCHES %3d", 1);

      if (ncr > 1)
	fprintf(fp, " %3ld", ncr - 1);
      if (nbo > 1)
	fprintf(fp, " %3ld", ncr*(nbo - 1));
      if (ntf > 1)
	fprintf(fp, " %3ld", ncr*(ntf - 1));
      if (ntm > 1)
	fprintf(fp, " %3ld", ncr*(ntm - 1));

      fprintf(fp, "\n");

      /* Pointer to first history */

      his = (long)RDB[lib + LIB_PTR_HIS];
      CheckPointer(FUNCTION_NAME, "(his)", DATA_ARRAY, his);
      
      /* Loop over data */

      xsp = (long)RDB[his + HIS_PTR_XS];
      while (xsp > VALID_PTR)
	{
	  /* Check burnup */

	  if ((long)RDB[xsp + XS_EXP_IDX] == 0)
	    {
	      fprintf(fp, "    %s  %2ld", GetText(xsp + XS_PARCS_STA_VAR),
		      (long)RDB[xsp + XS_PARCS_STA_IDX]);

	      if (ncr > 1)
		fprintf(fp, "%11.5f ", RDB[xsp + XS_CR]);
	      if (nbo > 1)
		fprintf(fp, "%11.5f ", RDB[xsp + XS_BOR]);
	      if (ntf > 1)
		fprintf(fp, "%11.5f ", RDB[xsp + XS_TFU]);
	      if (ntm > 1)
		fprintf(fp, "%11.5f ", RDB[xsp + XS_TMO]);
	      
	      fprintf(fp, "\n");
	    }

	  /* Next */
	  
	  xsp = NextItem(xsp);
	}

      /************************************************************************/

      /***** BURNUPS **********************************************************/

      /* Print title (NOTE: only one burnup set allowed) */

      fprintf(fp, "BURNUPS    1\n");

      /* Print values */

      fprintf(fp, "   1 %3ld", nbu);

      for (n = 0; n < nbu; n++)
	{
	  fprintf(fp, " %7.3f", EXP[n]);
	  if ((!((n + 1) % 8)) && (n != nbu - 1))
	    fprintf(fp, "\n        ");
	}
      
      fprintf(fp, "\n");

      /************************************************************************/

      /***** XS_SET ***********************************************************/

      /*  1. Series XS set series number */
      /*  2. IST Branches structure index */
      /*  3. NADF Number of ADF in each group. */
      /*  4. NCDF Number of CDF in each group. */
      /*  5. NCOLA Number of rod columns in whole assembly */
      /*  6. NROWA Number of rod rows in whole assembly */
      /*  7. NPART Index for computed part of assembly */
      /*  8. PITCH rod lattice pitch(cm) */
      /*  9. XBE start position for first column rods */
      /* 10. YBE start position for first row rods */
      /* 11. iHMD initial heavy metal density (g/cc) */
      /* 12. Dsat the saturated moderator density */
      /* 13. ARWatR the area ration of water rods to coolant */
      /* 14. ARByPa the area ration of bypass to coolant */
      /* 15. ARConR the area ration of control rods to coolant */

      /* Print title */

      fprintf(fp, "XS_SET  ");

      /* Print series number */

      fprintf(fp, " %7s", GetText(lib + LIB_PARCS_PTR_ID)); 

      /* Branch structure index */
      
      fprintf(fp, "  1");

      /* Number of ADF's and CDF's */

      if (flg & PARCS_FLAG_LADF)
	fprintf(fp, " %ld", (long)RDB[lib + LIB_INPUT_NSDF]);
      else
	fprintf(fp, " 0");

      if (flg & PARCS_FLAG_LCDF)
	fprintf(fp, " %ld", (long)RDB[lib + LIB_INPUT_NCDF]);
      else
	fprintf(fp, " 0");
      
      /* See 5.-7. above */
      
      fprintf(fp, " 0 0 0");

      /* Pitch */

      if (fiss == YES)
	fprintf(fp, " %7.5f", RDB[lib + LIB_PARCS_PITCH]);
      else
	fprintf(fp, " %7.5f", 0.0);

      /* See 9.-10. above */
  
      fprintf(fp, " 0.0 0.0");

      /* Heavy metal density and saturated moderator density */

      if (fiss == YES)
	fprintf(fp, " %7.5f %7.5f", RDB[lib + LIB_PARCS_IHMD],
		RDB[lib + LIB_PARCS_DSAT]);
      else
	fprintf(fp, " %7.5f %7.5f", 0.0, RDB[lib + LIB_PARCS_DSAT]);

      /* See 13.-15. above */
  
      fprintf(fp, " 0.0 0.0 0.0");

      fprintf(fp, "\n");

      /* Loop over histories (I guess?) */

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  /********************************************************************/

	  /***** HISTORYC *****************************************************/

	  /* Pointer to xs data */
	      
	  xsp = (long)RDB[his + HIS_PTR_XS];
	  CheckPointer(FUNCTION_NAME, "(xsp)", DATA_ARRAY, xsp);

	  /* Print invariant data */

	  fprintf(fp, "HISTORYC   1 %11.5f %11.5f %11.5f %11.5f\n", 
		  RDB[xsp + XS_CR], 
		  RDB[xsp + XS_BHI], 
		  RDB[xsp + XS_TFU], 
		  RDB[xsp + XS_TMO]);
	  
	  /* Loop over data */

	  while (xsp > 0.0)
	    {
	      /* Break loop if not reference case */

	      if (((long)RDB[xsp + XS_TMO_IDX] > 0) ||
		  ((long)RDB[xsp + XS_TFU_IDX] > 0) ||
		  ((long)RDB[xsp + XS_BOR_IDX] > 0) ||
		  ((long)RDB[xsp + XS_CR_IDX] > 0))
		break;

	      /* Prompt fission spectrum */

	      if (flg & PARCS_FLAG_LCHI)
		fprintf(fp, " %11.4e %11.4e", RDB[xsp + XS_CHIP1], 
			RDB[xsp + XS_CHIP2]);

	      /* Delayed fission spectrum */

	      if (flg & PARCS_FLAG_LCHD)
		fprintf(fp, " %11.4e %11.4e", RDB[xsp + XS_CHID1], 
			RDB[xsp + XS_CHID2]);

	      /* Inverse velocity */

	      if (flg & PARCS_FLAG_LINV)
		fprintf(fp, " %11.4e %11.4e", RDB[xsp + XS_INVV1], 
			RDB[xsp + XS_INVV2]);

	      /* Newline */

	      if ((flg & PARCS_FLAG_LCHI) || (flg & PARCS_FLAG_LCHD) ||
		  (flg & PARCS_FLAG_LINV))
		fprintf(fp, "\n");

	      /* Yield of I, Xe and Pm */

	      if (flg & PARCS_FLAG_LYLD)
		fprintf(fp, " %11.4e %11.4e %11.4e\n", RDB[xsp + XS_SIGI3],
			RDB[xsp + XS_SIGX3], RDB[xsp + XS_SIGP3]);
	      
	      /* Beta of delayed neutron */

	      if (flg & PARCS_FLAG_LBET)
		fprintf(fp, " %11.4e %11.4e %11.4e %11.4e %11.4e %11.4e\n", 
			RDB[xsp + XS_BETA1], RDB[xsp + XS_BETA2],
			RDB[xsp + XS_BETA3], RDB[xsp + XS_BETA4],
			RDB[xsp + XS_BETA5], RDB[xsp + XS_BETA6]);

	      /* Lambda of delayed neutron */

	      if (flg & PARCS_FLAG_LAMB)
		fprintf(fp, " %11.4e %11.4e %11.4e %11.4e %11.4e %11.4e\n", 
			RDB[xsp + XS_LBDA1], RDB[xsp + XS_LBDA2],
			RDB[xsp + XS_LBDA3], RDB[xsp + XS_LBDA4],
			RDB[xsp + XS_LBDA5], RDB[xsp + XS_LBDA6]);

	      /* Decay heat data */

	      if (flg & PARCS_FLAG_LDEC)
		Die(NULL, "Decay heat parameters not available (Ldec flag)");

	      /* Pointer to next */

	      xsp = NextItem(xsp);
	    }

	  /* Loop over data */

	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /****************************************************************/

	      /***** State identification *************************************/
	      
	      /* Check burnup index */

	      if (RDB[xsp + XS_EXP_IDX] == 0)
		{
		  /* Print name (viritelmä) */

		  if (!strcmp(GetText(xsp + XS_PARCS_STA_VAR), "RE"))
		    fprintf(fp, "REFERENC ");
		  else if (!strcmp(GetText(xsp + XS_PARCS_STA_VAR), "CR"))
		    fprintf(fp, "CRBRANCH ");
		  else if (!strcmp(GetText(xsp + XS_PARCS_STA_VAR), "PC"))
		    fprintf(fp, "SBBRANCH ");
		  else if (!strcmp(GetText(xsp + XS_PARCS_STA_VAR), "TF"))
		    fprintf(fp, "TFBRANCH ");
		  else if (!strcmp(GetText(xsp + XS_PARCS_STA_VAR), "TC"))
		    fprintf(fp, "TMBRANCH ");
		  else 
		    Die(FUNCTION_NAME, "Unidentified branch name %s",
			GetText(xsp + XS_PARCS_STA_VAR));

		  /* Print branch case and burnup set index */
		  
		  fprintf(fp, "%2ld   1\n", (long)RDB[xsp + XS_PARCS_STA_IDX]);
		}
	      
	      /****************************************************************/
	      
	      /***** XS Data block ********************************************/

	      /* Principal cross sections */
	     
 	      fprintf(fp, " %11.4E %11.4E %11.4E %11.4E", 
		      RDB[xsp + XS_TRANSP1], RDB[xsp + XS_TRANSP2],
		      RDB[xsp + XS_SIGA1], RDB[xsp + XS_SIGA2]);

	      fprintf(fp, " %11.4E %11.4E %11.4E %11.4E\n", 
		     RDB[xsp + XS_NUSF1], RDB[xsp + XS_NUSF2],
		      RDB[xsp + XS_EPSF1], RDB[xsp + XS_EPSF2]);
	      
	      /* Xenon & samarium (NOTE: in barns, fission is printed */
	      /* only if this flag is set) */

	      if (flg & PARCS_FLAG_LXES)
		fprintf(fp, " %11.4E %11.4E %11.4E %11.4E %11.4E %11.4E\n", 
			RDB[xsp + XS_SIGX1], RDB[xsp + XS_SIGX2],
			RDB[xsp + XS_SIGS1], RDB[xsp + XS_SIGS2],
			RDB[xsp + XS_SIGF1], RDB[xsp + XS_SIGF2]);
		
	      /* Detector response */

	      if (flg & PARCS_FLAG_LDET)
		Die(NULL, "Detector response not available (Ldet flag)");

	      /* Scattering xs */

	      fprintf(fp, " %11.4E %11.4E %11.4E %11.4E\n", 
		      RDB[xsp + XS_SIG11], RDB[xsp + XS_SIG21],
		      RDB[xsp + XS_SIG12], RDB[xsp + XS_SIG22]);

	      /* ADF's */

	      if (flg & PARCS_FLAG_LADF)
		{
		  /* Check maximum number of ADF's in input */

		  if ((n = (long)RDB[lib + LIB_NSDF]) == 4)
		    {	
		      /* Square geometry, check number of ADF's */

		      if ((long)RDB[lib + LIB_INPUT_NSDF] == 1)
			n = 1;
		      else if ((long)RDB[lib + LIB_INPUT_NSDF] != n)
			Die(NULL, "Invalid number of ADF's: %ld",
			    (long)RDB[lib + LIB_INPUT_NSDF]);

		      /* Print data */

		      for (i = 0; i < n; i++)
			fprintf(fp, " %11.4E", RDB[xsp + XS_SDF1W + i]);

		      for (i = 0; i < n; i++)
			fprintf(fp, " %11.4E", RDB[xsp + XS_SDF2W + i]);
		    }
		  else if ((n = (long)RDB[lib + LIB_NSDF]) == 6)
		    {	
		      /* Hex geometry, check number of ADF's */

		      if ((long)RDB[lib + LIB_INPUT_NSDF] == 1)
			n = 1;
		      else if ((long)RDB[lib + LIB_INPUT_NSDF] != n)
			Die(NULL, "Invalid number of ADF's: %ld",
			    (long)RDB[lib + LIB_INPUT_NSDF]);

		      /* Print data */

		      for (i = 0; i < n; i++)
			fprintf(fp, " %11.4E", RDB[xsp + XS_SDF11 + i]);

		      for (i = 0; i < n; i++)
			fprintf(fp, " %11.4E", RDB[xsp + XS_SDF21 + i]);
		    }

		  fprintf(fp, "\n");
		}

	      /* Direct energy deposition and J1 factors */

	      if (flg & PARCS_FLAG_LDED)
		Die(NULL, "Direct energy deposition not available (Lded flag)");

	      if (flg & PARCS_FLAG_LJ1F)
		Die(NULL, "J1 factors not available (Lj1f flag)");

	      /* CDF's */

	      if (flg & PARCS_FLAG_LCDF)
		{
		  /* Check maximum number of ADF's in input */

		  if ((n = (long)RDB[lib + LIB_NCDF]) == 4)
		    {	
		      /* Square geometry, check number of ADF's */

		      if ((long)RDB[lib + LIB_INPUT_NCDF] == 1)
			n = 1;
		      else if ((long)RDB[lib + LIB_INPUT_NCDF] != n)
			Die(NULL, "Invalid number of ADF's: %ld",
			    (long)RDB[lib + LIB_INPUT_NCDF]);

		      /* Print data */

		      for (i = 0; i < n; i++)
			fprintf(fp, " %11.4E", RDB[xsp + XS_CD1SW + i]);

		      for (i = 0; i < n; i++)
			fprintf(fp, " %11.4E", RDB[xsp + XS_CD2SW + i]);
		    }
		  else if ((n = (long)RDB[lib + LIB_NCDF]) == 6)
		    {	
		      /* Hex geometry, check number of ADF's */

		      if ((long)RDB[lib + LIB_INPUT_NCDF] == 1)
			n = 1;
		      else if ((long)RDB[lib + LIB_INPUT_NCDF] != n)
			Die(NULL, "Invalid number of ADF's: %ld",
			    (long)RDB[lib + LIB_INPUT_NCDF]);

		      /* Print data */

		      for (i = 0; i < n; i++)
			fprintf(fp, " %11.4E", RDB[xsp + XS_CDF11 + i]);

		      for (i = 0; i < n; i++)
			fprintf(fp, " %11.4E", RDB[xsp + XS_CDF21 + i]);
		    }

		  fprintf(fp, "\n");
		}

	      /* Group-wise form function */
	      
	      if (flg & PARCS_FLAG_LGFF)
		Die(NULL, "Form functions not available (Lgff flag)");

	      /****************************************************************/

	      /* Next */
	      
	      xsp = NextItem(xsp);
	    }
	  
	  /* Next history */

	  his = NextItem(his);
	}
    
      /* Close file */

      fclose(fp);

      /* Next library */

      lib = NextItem(lib);
    }

  /* Newline */

  printf("\n");
}

/*****************************************************************************/
