/*****************************************************************************/
/*                                                                           */
/* AresFit : readdata.c                                                      */
/*                                                                           */
/* Created:       2014/08/24 (JLe)                                           */
/* Last modified: 2016/11/01 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: Reads data from Serpent coe file                             */
/*                                                                           */
/* Comments: - Jos lisää uusia parametreja niin ne pitää poistaa tuolta      */
/*             vikasta if-haarasta                                           */
/*                                                                           */
/*****************************************************************************/

#include "header.h"
#include "locations.h"

#define FUNCTION_NAME "ReadData:"

long ArrayIdx(double *, double, long);

/*****************************************************************************/

void ReadData()
{
  long lib, his, np, xsp, ptr, nbu, i, ngcu, npa, sta_idx, n;
  double val, arr[1000], fiss1, fiss2, vhi, bhi, bor, cr, tfu, voi, tmo, rho;
  double bu, tmp;
  FILE *fp;
  char tmpstr[MAX_STR], dummy[MAX_STR], sta_var[MAX_STR];

  /* Loop over libraries */

  lib = (long)RDB[DATA_PTR_LIB0];
  while (lib > VALID_PTR)
    {
      /***********************************************************************/

      /***** Loop over history files *****************************************/

      /* Print */

      printf("Reading data for library '%s'...\n", 
	     GetText(lib + LIB_PTR_NAME));

      /* Loop over histories */

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  /*******************************************************************/
      
	  /***** Read data from file *****************************************/

	  /* Open file for reading */

	  fp = OpenFile(GetText(his + HIS_PTR_FILE), "r");

	  /* Copy library name */

	  WDB[his + HIS_PTR_LIB_NAME] = RDB[lib + LIB_PTR_NAME];

	  /* Avoid compiler warning */

	  xsp = -1;
	  
	  /* Loop over runs (first parameter is run index) */

	  while (fscanf(fp, "%ld", &i) != EOF)
	    {
	      /***************************************************************/

	      /***** Read header data ****************************************/

	      /* Read total number of runs and number of universes */

	      i = ReadI(fp);
	      i = ReadI(fp);
	      i = ReadI(fp);
	      ngcu = ReadI(fp);

	      /* Read number of branches */
	      
	      np = ReadI(fp);
	      
	      /* Skip values */
	      
	      for (i = 0; i < np; i++)
		if (fscanf(fp, "%s", tmpstr) == EOF)
		  Die(FUNCTION_NAME, "fscanf error");
	      
	      /* Reset variables */
	      
	      vhi = -INFTY;
	      bhi = -INFTY;
	      bor = -INFTY;
	      cr = -INFTY;
	      tfu = -INFTY;
	      voi = -INFTY;
	      tmo = -INFTY;
	      rho = -INFTY;

	      sta_var[0] = '\0';
	      sta_idx = -1;
	      
	      /* Read number of variables */
	      
	      np = ReadI(fp);

	      /* Loop over variables */
	      
	      for (i = 0; i < np; i++)
		{
		  /* Read name */
		  
		  if (fscanf(fp, "%s", tmpstr) == EOF)
		    Die(FUNCTION_NAME, "fscanf error");
		  
		  /* Read value */
		  
		  if (fscanf(fp, "%s", dummy) == EOF)
		    Die(FUNCTION_NAME, "fscanf error");
		  
		  /* PARCS variables */

		  if (!strcasecmp(tmpstr, "STA_VAR"))
		    strcpy(sta_var, dummy);
		  else if (!strcasecmp(tmpstr, "STA_IDX"))
		    sta_idx = AtoI(dummy);
		  else if ((!strcasecmp(tmpstr, "VERSION")) ||
			   (!strcasecmp(tmpstr, "DATE")) ||
			   (!strcasecmp(tmpstr, "TIME")))
		    {
		      /* Skip version, date and time */
		    }
		  else
		    {
		      /* Check that value is numerical */
		  
		      val = AtoF(dummy);
		      
		      /* Check */
		      
		      if (!strcasecmp(tmpstr, "VHI"))
			vhi = val;
		      else if (!strcasecmp(tmpstr, "BHI"))
			bhi = val;
		      else if (!strcasecmp(tmpstr, "BOR"))
			bor = val;
		      else if (!strcasecmp(tmpstr, "CR"))
			cr = val;
		      else if (!strcasecmp(tmpstr, "TFU"))
			tfu = val;
		      else if (!strcasecmp(tmpstr, "VOI"))
			voi = val;
		      else if (!strcasecmp(tmpstr, "TMO"))
			tmo = val;
		      else if (!strcasecmp(tmpstr, "RHO"))
			rho = val;
		    }
		}
	      
	      /* Read burnup */
	      
	      bu = ReadF(fp);
	      
	      /* Skip burnup index and total number of points */
	      
	      i = ReadI(fp);
	      i = ReadI(fp);

	      /***************************************************************/
	      
	      /**** Loop over universes **************************************/

	      while (ngcu-- > 0)
		{
		  /* Create new library */

		  xsp = NewItem(DATA_PTR_XS0, XS_BLOCK_SIZE);

		  /* Put library and hisotry pointers */

		  WDB[xsp + XS_PTR_LIB] = (double)lib;
		  WDB[xsp + XS_PTR_HIS] = (double)his;

		  /* Put PARCS variables */
		  
		  if ((long)RDB[DATA_CODE] == CODE_PARCS)
		    {
		      if (sta_var[0] == '\0')
			Die(0, "STA_VAR not set");
		      else
			WDB[xsp + XS_PARCS_STA_VAR] = (double)PutText(sta_var);
		      
		      WDB[xsp + XS_PARCS_STA_IDX] = (double)sta_idx;
		    }

		  /* Void / moderator temperature history */
		  
		  if (vhi > -INFTY)
		    {
		      WDB[xsp + XS_VHI] = vhi;
		      WDB[lib + LIB_PARA_VHI] = (double)YES;
		    }
		  
		  /* Boron history */
		  
		  if (bhi > -INFTY)
		    {
		      WDB[xsp + XS_BHI] = bhi;
		      WDB[lib + LIB_PARA_BHI] = (double)YES;
		    }
		  
		  /* Boron branch */
		  
		  if (bor > -INFTY)
		    {
		      WDB[xsp + XS_BOR] = bor;
		      WDB[lib + LIB_PARA_BOR] = (double)YES;
		    }
		  
		  /* Control rod branch */
		  
		  if (cr > -INFTY)
		    {
		      WDB[xsp + XS_CR] = cr;
		      WDB[lib + LIB_PARA_CR] = (double)YES;
		    }
		  
		  /* Fuel temperature branch */
		  
		  if (tfu > -INFTY)
		    {
		      WDB[xsp + XS_TFU] = tfu;
		      WDB[lib + LIB_PARA_TFU] = (double)YES;
		    }
		  
		  /* Void branch */
		  
		  if (voi > -INFTY)
		    {
		      WDB[xsp + XS_VOI] = voi;
		      WDB[lib + LIB_PARA_VOI] = (double)YES;
		    }
		  
		  /* Moderator temperature branch */
		  
		  if (tmo > -INFTY)
		    {
		      WDB[xsp + XS_TMO] = tmo;
		      WDB[lib + LIB_PARA_TMO] = (double)YES;
		    }
		  
		  /* Moderator density branch */
		  
		  if (rho > -INFTY)
		    {
		      WDB[xsp + XS_RHO] = rho;
		      WDB[lib + LIB_PARA_RHO] = (double)YES;
		    }

		  /* put burnup */
		  
		  WDB[xsp + XS_EXP] = bu;
		  
		  /* Read universe name and number of parameters */
		  
		  if (fscanf(fp, "%s %ld", tmpstr, &npa) == EOF)
		    Die(FUNCTION_NAME, "fscaf error");
		  
		  /* Put universe name */

		  WDB[xsp + XS_PTR_GCU] = PutText(tmpstr);

		  /* Loop over parameters */
		  
		  while (npa-- > 0)
		    {
		      /* Read name */

		      if (fscanf(fp, "%s", tmpstr) == EOF)
			Die(FUNCTION_NAME, "fscaf error");
		      
		      /* Check parameter */
		      
		      if ((!strcmp(tmpstr, "INF_RABSXS")) ||
			  (!strcmp(tmpstr, "B1_RABSXS")))
			{
			  /* Read number of values */
			  
			  np = ReadI(fp);
			  
			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");
			    
			  /* Read data */
			  
			  WDB[xsp + XS_SIGA1] = ReadF(fp);
			  WDB[xsp + XS_SIGA2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_NSF")) ||
			       (!strcmp(tmpstr, "B1_NSF")))
			{
			  /* Read number of values */
			  
			  np = ReadI(fp);

			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");
			  
			  /* Read data */
			  
			  WDB[xsp + XS_NUSF1] = ReadF(fp);
			  WDB[xsp + XS_NUSF2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_CHIP")) ||
			       (!strcmp(tmpstr, "B1_CHIP")))
			{
			  /* Read number of values */
			  
			  np = ReadI(fp);

			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");
			  
			  /* Read data */
			  
			  WDB[xsp + XS_CHIP1] = ReadF(fp);
			  WDB[xsp + XS_CHIP2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_CHID")) ||
			       (!strcmp(tmpstr, "B1_CHID")))
			{
			  /* Read number of values */
			  
			  np = ReadI(fp);

			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");
			  
			  /* Read data */
			  
			  WDB[xsp + XS_CHID1] = ReadF(fp);
			  WDB[xsp + XS_CHID2] = ReadF(fp);
			}
		      else if (!strcmp(tmpstr, "B1_TRANSPXS"))
			{
			  /* Read number of values */
			  
			  np = ReadI(fp);

			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");
			  
			  printf("Huom! luetaan B1_TRANSXS\n");

			  /* Read data */
			  
			  WDB[xsp + XS_TRANSP1] = ReadF(fp);
			  WDB[xsp + XS_TRANSP2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_DIFFCOEF")) ||
			       (!strcmp(tmpstr, "B1_DIFFCOEF")))
			{
			  /* Read number of values */
			  
			  np = ReadI(fp);

			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");
			  
			  /* Read data */
			  
			  WDB[xsp + XS_DIFF1] = ReadF(fp);
			  WDB[xsp + XS_DIFF2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_S0")) ||
			       (!strcmp(tmpstr, "B1_S0")))
			{
			  /* Read number of values */
			  
			  np = ReadI(fp);
			  
			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] != 0)
			    if (RDB[lib + LIB_NEG] != sqrt((double)np))
			      Die(NULL, "Error in number of energy groups");

			  /* Read data (järjestys muutettu 2.1.25) */

			  if (1 == 2)
			    {
			      WDB[xsp + XS_SIG11] = ReadF(fp);
			      WDB[xsp + XS_SIG21] = ReadF(fp);
			      WDB[xsp + XS_SIG12] = ReadF(fp);
			      WDB[xsp + XS_SIG22] = ReadF(fp);
			    }
			  else
			    {
			      WDB[xsp + XS_SIG11] = ReadF(fp);
			      WDB[xsp + XS_SIG12] = ReadF(fp);
			      WDB[xsp + XS_SIG21] = ReadF(fp);
			      WDB[xsp + XS_SIG22] = ReadF(fp);
			    }
			}
		      else if ((!strcmp(tmpstr, "INF_FLX")) ||
			       (!strcmp(tmpstr, "B1_FLX")))
			{
			  /* Read number of values */
			  
			  np = ReadI(fp);

			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");
			  
			  /* Read data */
			  
			  WDB[xsp + XS_FLUX1] = ReadF(fp);
			  WDB[xsp + XS_FLUX2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_KAPPA")) ||
			       (!strcmp(tmpstr, "B1_KAPPA")))
			{
			  /* Read number of values */
			  
			  np = ReadI(fp);

			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");
			  
			  /* Read data */
			  
			  WDB[xsp + XS_EPSF1] = ReadF(fp);
			  WDB[xsp + XS_EPSF2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_FISS")) ||
			       (!strcmp(tmpstr, "B1_FISS")))
			{
			  /* Read number of values */
			  
			  np = ReadI(fp);
			  
			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");

			  /* Read data */
			  
			  WDB[xsp + XS_SIGF1] = ReadF(fp);
			  WDB[xsp + XS_SIGF2] = ReadF(fp);
			}
		      else if (!strcmp(tmpstr, "DF_SURF_DF"))
			{
			  /* Read number of values */
			  
			  np = ReadI(fp);
			  
			  /* Check with number of ADF's */

			  if ((long)RDB[lib + LIB_NSDF] == 0)
			    WDB[lib + LIB_NSDF] = (double)np;
			  else if ((long)RDB[lib + LIB_NSDF] != np)
			    Die(NULL, "Error in number of surface ADF's");

			  /* Check */
			  
			  if (((long)RDB[lib + LIB_NOMCON_NHET] > 0) &&
			      (np % (long)RDB[lib + LIB_NOMCON_NHET]))
			    Die(NULL, "Inconsistent number of ADF surfaces");
			  
			  /* Read data */
			  
			  if (np == 8)
			    {
			      /* Järjestys muutettu 2.1.25 */

			      if (1 == 2)
				{
				  WDB[xsp + XS_SDF1W] = ReadF(fp);
				  WDB[xsp + XS_SDF1S] = ReadF(fp);
				  WDB[xsp + XS_SDF1E] = ReadF(fp);
				  WDB[xsp + XS_SDF1N] = ReadF(fp);
				  WDB[xsp + XS_SDF2W] = ReadF(fp);
				  WDB[xsp + XS_SDF2S] = ReadF(fp);
				  WDB[xsp + XS_SDF2E] = ReadF(fp);
				  WDB[xsp + XS_SDF2N] = ReadF(fp);
				}
			      else
				{
				  WDB[xsp + XS_SDF1W] = ReadF(fp);
				  WDB[xsp + XS_SDF2W] = ReadF(fp);
				  WDB[xsp + XS_SDF1S] = ReadF(fp);
				  WDB[xsp + XS_SDF2S] = ReadF(fp);
				  WDB[xsp + XS_SDF1E] = ReadF(fp);
				  WDB[xsp + XS_SDF2E] = ReadF(fp);
				  WDB[xsp + XS_SDF1N] = ReadF(fp);
				  WDB[xsp + XS_SDF2N] = ReadF(fp);
				}

			      /* Check values */
			      
			      if ((RDB[xsp + XS_SDF1W] != 0.0) &&
				  ((RDB[xsp + XS_SDF1W] < 0.4) ||
				   (RDB[xsp + XS_SDF1W] > 4.0)))
				printf("Suspicious ADF value %1.5f (W, 1)\n",
				       RDB[xsp + XS_SDF1W]);
			      
			      if ((RDB[xsp + XS_SDF2W] != 0.0) &&
				  ((RDB[xsp + XS_SDF2W] < 0.4) ||
				   (RDB[xsp + XS_SDF2W] > 4.0)))
				printf("Suspicious ADF value %1.5f (W, 2)\n",
				       RDB[xsp + XS_SDF2W]);
			  
			      if ((RDB[xsp + XS_SDF1S] != 0.0) &&
				  ((RDB[xsp + XS_SDF1S] < 0.4) ||
				   (RDB[xsp + XS_SDF1S] > 4.0)))
				printf("Suspicious ADF value %1.5f (S, 1)\n",
				       RDB[xsp + XS_SDF1S]);
			      
			      if ((RDB[xsp + XS_SDF2S] != 0.0) &&
				  ((RDB[xsp + XS_SDF2S] < 0.4) ||
				   (RDB[xsp + XS_SDF2S] > 4.0)))
				printf("Suspicious ADF value %1.5f (S, 2)\n",
				       RDB[xsp + XS_SDF2S]);
			      
			      if ((RDB[xsp + XS_SDF1E] != 0.0) &&
				  ((RDB[xsp + XS_SDF1E] < 0.4) ||
				   (RDB[xsp + XS_SDF1E] > 4.0)))
				printf("Suspicious ADF value %1.5f (E, 1)\n",
				       RDB[xsp + XS_SDF1E]);
			      
			      if ((RDB[xsp + XS_SDF2E] != 0.0) &&
				  ((RDB[xsp + XS_SDF2E] < 0.4) ||
				   (RDB[xsp + XS_SDF2E] > 4.0)))
				printf("Suspicious ADF value %1.5f (E, 2)\n",
				       RDB[xsp + XS_SDF2E]);
			      
			      if ((RDB[xsp + XS_SDF1N] != 0.0) &&
				  ((RDB[xsp + XS_SDF1N] < 0.4) ||
				   (RDB[xsp + XS_SDF1N] > 4.0)))
				printf("Suspicious ADF value %1.5f (N, 1)\n",
				       RDB[xsp + XS_SDF1N]);
			      
			      if ((RDB[xsp + XS_SDF2N] != 0.0) &&
				  ((RDB[xsp + XS_SDF2N] < 0.4) ||
				   (RDB[xsp + XS_SDF2N] > 4.0)))
				printf("Suspicious ADF value %1.5f (N, 2)\n",
				       RDB[xsp + XS_SDF2N]);
			    }
			  else if (np == 12)
			    {
			      /* Järjestys muutettu 2.1.25 */

			      if (1 == 2)
				{
				  WDB[xsp + XS_SDF11] = ReadF(fp);
				  WDB[xsp + XS_SDF12] = ReadF(fp);
				  WDB[xsp + XS_SDF13] = ReadF(fp);
				  WDB[xsp + XS_SDF14] = ReadF(fp);
				  WDB[xsp + XS_SDF15] = ReadF(fp);
				  WDB[xsp + XS_SDF16] = ReadF(fp);
				  WDB[xsp + XS_SDF21] = ReadF(fp);
				  WDB[xsp + XS_SDF22] = ReadF(fp);
				  WDB[xsp + XS_SDF23] = ReadF(fp);
				  WDB[xsp + XS_SDF24] = ReadF(fp);
				  WDB[xsp + XS_SDF25] = ReadF(fp);
				  WDB[xsp + XS_SDF26] = ReadF(fp);
				}
			      else
				{
				  WDB[xsp + XS_SDF11] = ReadF(fp);
				  WDB[xsp + XS_SDF21] = ReadF(fp);
				  WDB[xsp + XS_SDF12] = ReadF(fp);
				  WDB[xsp + XS_SDF22] = ReadF(fp);
				  WDB[xsp + XS_SDF13] = ReadF(fp);
				  WDB[xsp + XS_SDF23] = ReadF(fp);
				  WDB[xsp + XS_SDF14] = ReadF(fp);
				  WDB[xsp + XS_SDF24] = ReadF(fp);
				  WDB[xsp + XS_SDF15] = ReadF(fp);
				  WDB[xsp + XS_SDF25] = ReadF(fp);
				  WDB[xsp + XS_SDF16] = ReadF(fp);
				  WDB[xsp + XS_SDF26] = ReadF(fp);
				}

			      /* Check values */
			      
			      if ((RDB[xsp + XS_SDF11] != 0.0) &&
				  ((RDB[xsp + XS_SDF11] < 0.4) ||
				   (RDB[xsp + XS_SDF11] > 4.0)))
				printf("Suspicious ADF value %1.5f (1, 1)\n",
				       RDB[xsp + XS_SDF11]);
			      
			      if ((RDB[xsp + XS_SDF21] != 0.0) &&
				  ((RDB[xsp + XS_SDF21] < 0.4) ||
				   (RDB[xsp + XS_SDF21] > 4.0)))
				printf("Suspicious ADF value %1.5f (1, 2)\n",
				       RDB[xsp + XS_SDF21]);
			  
			      if ((RDB[xsp + XS_SDF12] != 0.0) &&
				  ((RDB[xsp + XS_SDF12] < 0.4) ||
				   (RDB[xsp + XS_SDF12] > 4.0)))
				printf("Suspicious ADF value %1.5f (2, 1)\n",
				       RDB[xsp + XS_SDF12]);
			      
			      if ((RDB[xsp + XS_SDF22] != 0.0) &&
				  ((RDB[xsp + XS_SDF22] < 0.4) ||
				   (RDB[xsp + XS_SDF22] > 4.0)))
				printf("Suspicious ADF value %1.5f (2, 2)\n",
				       RDB[xsp + XS_SDF22]);
			      
			      if ((RDB[xsp + XS_SDF13] != 0.0) &&
				  ((RDB[xsp + XS_SDF13] < 0.4) ||
				   (RDB[xsp + XS_SDF13] > 4.0)))
				printf("Suspicious ADF value %1.5f (3, 1)\n",
				       RDB[xsp + XS_SDF13]);
			      
			      if ((RDB[xsp + XS_SDF23] != 0.0) &&
				  ((RDB[xsp + XS_SDF23] < 0.4) ||
				   (RDB[xsp + XS_SDF23] > 4.0)))
				printf("Suspicious ADF value %1.5f (3, 2)\n",
				       RDB[xsp + XS_SDF23]);
			      
			      if ((RDB[xsp + XS_SDF14] != 0.0) &&
				  ((RDB[xsp + XS_SDF14] < 0.4) ||
				   (RDB[xsp + XS_SDF14] > 4.0)))
				printf("Suspicious ADF value %1.5f (4, 1)\n",
				       RDB[xsp + XS_SDF14]);
			      
			      if ((RDB[xsp + XS_SDF24] != 0.0) &&
				  ((RDB[xsp + XS_SDF24] < 0.4) ||
				   (RDB[xsp + XS_SDF24] > 4.0)))
				printf("Suspicious ADF value %1.5f (4, 2)\n",
				       RDB[xsp + XS_SDF24]);

			      if ((RDB[xsp + XS_SDF15] != 0.0) &&
				  ((RDB[xsp + XS_SDF15] < 0.4) ||
				   (RDB[xsp + XS_SDF15] > 4.0)))
				printf("Suspicious ADF value %1.5f (5, 1)\n",
				       RDB[xsp + XS_SDF15]);
			      
			      if ((RDB[xsp + XS_SDF25] != 0.0) &&
				  ((RDB[xsp + XS_SDF25] < 0.4) ||
				   (RDB[xsp + XS_SDF25] > 4.0)))
				printf("Suspicious ADF value %1.5f (5, 2)\n",
				       RDB[xsp + XS_SDF25]);

			      if ((RDB[xsp + XS_SDF16] != 0.0) &&
				  ((RDB[xsp + XS_SDF16] < 0.4) ||
				   (RDB[xsp + XS_SDF16] > 4.0)))
				printf("Suspicious ADF value %1.5f (6, 1)\n",
				       RDB[xsp + XS_SDF16]);
			      
			      if ((RDB[xsp + XS_SDF26] != 0.0) &&
				  ((RDB[xsp + XS_SDF26] < 0.4) ||
				   (RDB[xsp + XS_SDF26] > 4.0)))
				printf("Suspicious ADF value %1.5f (6, 2)\n",
				       RDB[xsp + XS_SDF26]);
			    }
			  else
			    Die(FUNCTION_NAME, "np = %ld", np);
			}
		      else if (!strcmp(tmpstr, "DF_CORN_DF"))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);

			  /* Check with number of ADF's */

			  if ((long)RDB[lib + LIB_NCDF] == 0)
			    WDB[lib + LIB_NCDF] = (double)np;
			  else if ((long)RDB[lib + LIB_NCDF] != np)
			    Die(NULL, "Error in number of corner ADF's");
		      
			  /* Read data */
			  
			  if (np == 8)
			    {
			      /* Järjestys muutettu 2.1.25 */

			      if (1 == 2)
				{
				  WDB[xsp + XS_CD1NW] = ReadF(fp);
				  WDB[xsp + XS_CD1NE] = ReadF(fp);
				  WDB[xsp + XS_CD1SE] = ReadF(fp);
				  WDB[xsp + XS_CD1SW] = ReadF(fp);
				  WDB[xsp + XS_CD2NW] = ReadF(fp);
				  WDB[xsp + XS_CD2NE] = ReadF(fp);
				  WDB[xsp + XS_CD2SE] = ReadF(fp);
				  WDB[xsp + XS_CD2SW] = ReadF(fp);
				}
			      else
				{
				  WDB[xsp + XS_CD1NW] = ReadF(fp);
				  WDB[xsp + XS_CD2NW] = ReadF(fp);
				  WDB[xsp + XS_CD1NE] = ReadF(fp);
				  WDB[xsp + XS_CD2NE] = ReadF(fp);
				  WDB[xsp + XS_CD1SE] = ReadF(fp);
				  WDB[xsp + XS_CD2SE] = ReadF(fp);
				  WDB[xsp + XS_CD1SW] = ReadF(fp);
				  WDB[xsp + XS_CD2SW] = ReadF(fp);
				}

			      /* Check values */
			      
			      if ((RDB[xsp + XS_CD1NW] != 0.0) &&
				  ((RDB[xsp + XS_CD1NW] < 0.4) ||
				   (RDB[xsp + XS_CD1NW] > 4.0)))
				printf("Suspicious ADF value %1.5f (NW, 1)\n",
				       RDB[xsp + XS_CD1NW]);
			      
			      if ((RDB[xsp + XS_CD2NW] != 0.0) &&
				  ((RDB[xsp + XS_CD2NW] < 0.4) ||
				   (RDB[xsp + XS_CD2NW] > 4.0)))
				printf("Suspicious ADF value %1.5f (NW, 2)\n",
				       RDB[xsp + XS_CD2NW]);
			      
			      if ((RDB[xsp + XS_CD1NE] != 0.0) &&
				  ((RDB[xsp + XS_CD1NE] < 0.4) ||
				   (RDB[xsp + XS_CD1NE] > 4.0)))
				printf("Suspicious ADF value %1.5f (NE, 1)\n",
				       RDB[xsp + XS_CD1NE]);
			      
			      if ((RDB[xsp + XS_CD2NE] != 0.0) &&
				  ((RDB[xsp + XS_CD2NE] < 0.4) ||
				   (RDB[xsp + XS_CD2NE] > 4.0)))
				printf("Suspicious ADF value %1.5f (NE, 2)\n",
				       RDB[xsp + XS_CD2NE]);
			      
			      if ((RDB[xsp + XS_CD1SE] != 0.0) &&
				  ((RDB[xsp + XS_CD1SE] < 0.4) ||
				   (RDB[xsp + XS_CD1SE] > 4.0)))
				printf("Suspicious ADF value %1.5f (SE, 1)\n",
				       RDB[xsp + XS_CD1SE]);
			      
			      if ((RDB[xsp + XS_CD2SE] != 0.0) &&
				  ((RDB[xsp + XS_CD2SE] < 0.4) ||
				   (RDB[xsp + XS_CD2SE] > 4.0)))
				printf("Suspicious ADF value %1.5f (SE, 2)\n",
				       RDB[xsp + XS_CD2SE]);
			      
			      if ((RDB[xsp + XS_CD1SW] != 0.0) &&
				  ((RDB[xsp + XS_CD1SW] < 0.4) ||
				   (RDB[xsp + XS_CD1SW] > 4.0)))
				printf("Suspicious ADF value %1.5f (SW, 1)\n",
				       RDB[xsp + XS_CD1SW]);
			      
			      if ((RDB[xsp + XS_CD2SW] != 0.0) &&
				  ((RDB[xsp + XS_CD2SW] < 0.4) ||
				   (RDB[xsp + XS_CD2SW] > 4.0)))
				printf("Suspicious ADF value %1.5f (SW, 2)\n",
				       RDB[xsp + XS_CD2SW]);
			    }
			  else if (np == 12)
			    {
			      /* Järjestys muutettu 2.1.25 */

			      if (1 == 2)
				{
				  WDB[xsp + XS_CDF11] = ReadF(fp);
				  WDB[xsp + XS_CDF12] = ReadF(fp);
				  WDB[xsp + XS_CDF13] = ReadF(fp);
				  WDB[xsp + XS_CDF14] = ReadF(fp);
				  WDB[xsp + XS_CDF15] = ReadF(fp);
				  WDB[xsp + XS_CDF16] = ReadF(fp);
				  WDB[xsp + XS_CDF21] = ReadF(fp);
				  WDB[xsp + XS_CDF22] = ReadF(fp);
				  WDB[xsp + XS_CDF23] = ReadF(fp);
				  WDB[xsp + XS_CDF24] = ReadF(fp);
				  WDB[xsp + XS_CDF25] = ReadF(fp);
				  WDB[xsp + XS_CDF26] = ReadF(fp);
				}
			      else
				{
				  WDB[xsp + XS_CDF11] = ReadF(fp);
				  WDB[xsp + XS_CDF21] = ReadF(fp);
				  WDB[xsp + XS_CDF12] = ReadF(fp);
				  WDB[xsp + XS_CDF22] = ReadF(fp);
				  WDB[xsp + XS_CDF13] = ReadF(fp);
				  WDB[xsp + XS_CDF23] = ReadF(fp);
				  WDB[xsp + XS_CDF14] = ReadF(fp);
				  WDB[xsp + XS_CDF24] = ReadF(fp);
				  WDB[xsp + XS_CDF15] = ReadF(fp);
				  WDB[xsp + XS_CDF25] = ReadF(fp);
				  WDB[xsp + XS_CDF16] = ReadF(fp);
				  WDB[xsp + XS_CDF26] = ReadF(fp);
				}
						      
			      /* Check values */
			      
			      if ((RDB[xsp + XS_CDF11] != 0.0) &&
				  ((RDB[xsp + XS_CDF11] < 0.4) ||
				   (RDB[xsp + XS_CDF11] > 4.0)))
				printf("Suspicious ADF value %1.5f (1, 1)\n",
				       RDB[xsp + XS_CDF11]);
			      
			      if ((RDB[xsp + XS_CDF21] != 0.0) &&
				  ((RDB[xsp + XS_CDF21] < 0.4) ||
				   (RDB[xsp + XS_CDF21] > 4.0)))
				printf("Suspicious ADF value %1.5f (1, 2)\n",
				       RDB[xsp + XS_CDF21]);
			  
			      if ((RDB[xsp + XS_CDF12] != 0.0) &&
				  ((RDB[xsp + XS_CDF12] < 0.4) ||
				   (RDB[xsp + XS_CDF12] > 4.0)))
				printf("Suspicious ADF value %1.5f (2, 1)\n",
				       RDB[xsp + XS_CDF12]);
			      
			      if ((RDB[xsp + XS_CDF22] != 0.0) &&
				  ((RDB[xsp + XS_CDF22] < 0.4) ||
				   (RDB[xsp + XS_CDF22] > 4.0)))
				printf("Suspicious ADF value %1.5f (2, 2)\n",
				       RDB[xsp + XS_CDF22]);
			      
			      if ((RDB[xsp + XS_CDF13] != 0.0) &&
				  ((RDB[xsp + XS_CDF13] < 0.4) ||
				   (RDB[xsp + XS_CDF13] > 4.0)))
				printf("Suspicious ADF value %1.5f (3, 1)\n",
				       RDB[xsp + XS_CDF13]);
			      
			      if ((RDB[xsp + XS_CDF23] != 0.0) &&
				  ((RDB[xsp + XS_CDF23] < 0.4) ||
				   (RDB[xsp + XS_CDF23] > 4.0)))
				printf("Suspicious ADF value %1.5f (3, 2)\n",
				       RDB[xsp + XS_CDF23]);
			      
			      if ((RDB[xsp + XS_CDF14] != 0.0) &&
				  ((RDB[xsp + XS_CDF14] < 0.4) ||
				   (RDB[xsp + XS_CDF14] > 4.0)))
				printf("Suspicious ADF value %1.5f (4, 1)\n",
				       RDB[xsp + XS_CDF14]);
			      
			      if ((RDB[xsp + XS_CDF24] != 0.0) &&
				  ((RDB[xsp + XS_CDF24] < 0.4) ||
				   (RDB[xsp + XS_CDF24] > 4.0)))
				printf("Suspicious ADF value %1.5f (4, 2)\n",
				       RDB[xsp + XS_CDF24]);

			      if ((RDB[xsp + XS_CDF15] != 0.0) &&
				  ((RDB[xsp + XS_CDF15] < 0.4) ||
				   (RDB[xsp + XS_CDF15] > 4.0)))
				printf("Suspicious ADF value %1.5f (5, 1)\n",
				       RDB[xsp + XS_CDF15]);
			      
			      if ((RDB[xsp + XS_CDF25] != 0.0) &&
				  ((RDB[xsp + XS_CDF25] < 0.4) ||
				   (RDB[xsp + XS_CDF25] > 4.0)))
				printf("Suspicious ADF value %1.5f (5, 2)\n",
				       RDB[xsp + XS_CDF25]);

			      if ((RDB[xsp + XS_CDF16] != 0.0) &&
				  ((RDB[xsp + XS_CDF16] < 0.4) ||
				   (RDB[xsp + XS_CDF16] > 4.0)))
				printf("Suspicious ADF value %1.5f (6, 1)\n",
				       RDB[xsp + XS_CDF16]);
			      
			      if ((RDB[xsp + XS_CDF26] != 0.0) &&
				  ((RDB[xsp + XS_CDF26] < 0.4) ||
				   (RDB[xsp + XS_CDF26] > 4.0)))
				printf("Suspicious ADF value %1.5f (6, 2)\n",
				       RDB[xsp + XS_CDF26]);
			    }
			  else
			    Die(FUNCTION_NAME, "np = %ld", np);
			}
		      else if ((!strcmp(tmpstr, "INF_I135_MICRO_ABS")) ||
			       (!strcmp(tmpstr, "B1_I135_MICRO_ABS")))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);

			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");
		      
			  /* Read data */
		      
			  WDB[xsp + XS_SIGI1] = ReadF(fp);
			  WDB[xsp + XS_SIGI2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_I135_YIELD")) ||
			       (!strcmp(tmpstr, "B1_I135_YIELD")))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);
		      
			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");

			  /* Read data */
		      
			  WDB[xsp + XS_YLD_I1] = ReadF(fp);
			  WDB[xsp + XS_YLD_I2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_XE135_MICRO_ABS")) ||
			       (!strcmp(tmpstr, "B1_XE135_MICRO_ABS")))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);
		      
			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");

			  /* Read data */
		      
			  WDB[xsp + XS_SIGX1] = ReadF(fp);
			  WDB[xsp + XS_SIGX2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_XE135_MACRO_ABS")) ||
			       (!strcmp(tmpstr, "B1_XE135_MACRO_ABS")))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);
		      
			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");

			  /* Read data */
		      
			  WDB[xsp + XS_MACRO_XE1] = ReadF(fp);
			  WDB[xsp + XS_MACRO_XE2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_XE135_YIELD")) ||
			       (!strcmp(tmpstr, "B1_XE135_YIELD")))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);
		      
			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");

			  /* Read data */
		      
			  WDB[xsp + XS_YLD_XE1] = ReadF(fp);
			  WDB[xsp + XS_YLD_XE2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_PM149_MICRO_ABS")) ||
			       (!strcmp(tmpstr, "B1_PM149_MICRO_ABS")))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);

			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");
		      
			  /* Read data */
		      
			  WDB[xsp + XS_SIGP1] = ReadF(fp);
			  WDB[xsp + XS_SIGP2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_PM149_YIELD")) ||
			       (!strcmp(tmpstr, "B1_PM149_YIELD")))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);

			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");
		      
			  /* Read data */
		      
			  WDB[xsp + XS_YLD_PM1] = ReadF(fp);
			  WDB[xsp + XS_YLD_PM2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_SM149_MICRO_ABS")) ||
			       (!strcmp(tmpstr, "B1_SM149_MICRO_ABS")))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);
		      
			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");

			  /* Read data */
		      
			  WDB[xsp + XS_SIGS1] = ReadF(fp);
			  WDB[xsp + XS_SIGS2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_SM149_MACRO_ABS")) ||
			       (!strcmp(tmpstr, "B1_SM149_MACRO_ABS")))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);
		      
			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");

			  /* Read data */
		      
			  WDB[xsp + XS_MACRO_SM1] = ReadF(fp);
			  WDB[xsp + XS_MACRO_SM2] = ReadF(fp);
			}
		      else if ((!strcmp(tmpstr, "INF_SM149_YIELD")) ||
			       (!strcmp(tmpstr, "B1_SM149_YIELD")))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);
		      
			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");

			  /* Read data */
		      
			  WDB[xsp + XS_YLD_SM1] = ReadF(fp);
			  WDB[xsp + XS_YLD_SM2] = ReadF(fp);
			}
		      else if (!strcmp(tmpstr, "BETA_EFF"))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);

			  /* Read data */

			  n = 0;
			  for (i = 0; i < np; i++)
			    if ((WDB[xsp + XS_BETA + i] = ReadF(fp)) > 0.0)
			      n++;

			  /* Check maximum */

			  if (n > 9)
			    Die(FUNCTION_NAME, 
				"Number of precursor groups exceeds max");

			  /* Check with number of precursor groups */
			  
			  if (n > 0)
			    {
			      if ((long)RDB[lib + LIB_NDG] == 0)
				WDB[lib + LIB_NDG] = (double)(n - 1);
			      else if ((long)RDB[lib + LIB_NDG] != (n - 1))
				Die(NULL, 
				    "Error in number of prefcursor groups");
			    }
			}
		      else if ((!strcmp(tmpstr, "INF_INVV")) ||
			       (!strcmp(tmpstr, "B1_INVV")))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);
		      
			  /* Check with number of energy groups */

			  if ((long)RDB[lib + LIB_NEG] == 0)
			    WDB[lib + LIB_NEG] = (double)np;
			  else if ((long)RDB[lib + LIB_NEG] != np)
			    Die(NULL, "Error in number of energy groups");

			  /* Read data */
		      
			  WDB[xsp + XS_INVV1] = ReadF(fp);
			  WDB[xsp + XS_INVV2] = ReadF(fp);
			}
		      else if (!strcmp(tmpstr, "LAMBDA"))
			{
			  /* Read number of values */
		      
			  np = ReadI(fp);

			  /* Read data */

			  n = 0;
			  for (i = 0; i < np; i++)
			    if ((WDB[xsp + XS_LBDA + i] = ReadF(fp)) > 0.0)
			      n++;

			  /* Check maximum */

			  if (n > 9)
			    Die(FUNCTION_NAME, 
				"Number of precursor groups exceeds max");

			  /* Check with number of precursor groups */

			  if (n > 0)
			    {
			      if ((long)RDB[lib + LIB_NDG] == 0)
				WDB[lib + LIB_NDG] = (double)(n - 1);
			      else if ((long)RDB[lib + LIB_NDG] != (n - 1))
				Die(NULL, 
				    "Error in number of precursor groups");
			    }
			}
		      else
			{
			  /* Unrecognized parameter */
			  /*
			  printf("Unrecognized parameter \"%s\"\n", tmpstr);
			  */
			  /* Read number of values */

			  np = ReadI(fp);

			  /* Skip dummy values */
		      
			  for (i = 0; i < np; i++)
			    ReadF(fp);
			}
		    }
	    
		  /************************************************************/
		}
	    }
	
	  /* Close file */

	  fclose(fp);
	  
	  /* Next history */

	  his = NextItem(his);
	}

      /***********************************************************************/

      /* Check energy groups */

      if ((long)RDB[lib + LIB_NEG] < 1)
	Die(NULL, "Number of energy groups not set");

      /* Convert DF's */

      WDB[lib + LIB_NSDF] = RDB[lib + LIB_NSDF]/RDB[lib + LIB_NEG];
      WDB[lib + LIB_NCDF] = RDB[lib + LIB_NCDF]/RDB[lib + LIB_NEG];

      /* Next library */
      
      lib = NextItem(lib);
    }

  /* Reconfigure libraries */

  Reconfigure();

  /* Loop over libraries */

  lib = (long)RDB[DATA_PTR_LIB0];
  while (lib > VALID_PTR)
    {
      /***********************************************************************/

      /***** Loop over history files *****************************************/

      /* Loop over histories */

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  /*******************************************************************/

	  /***** Convert some cross sections *********************************/
	
	  /* Loop over data */

	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Calculate fission rates */

	      fiss1 = RDB[xsp + XS_FLUX1]*RDB[xsp + XS_SIGF1];
	      fiss2 = RDB[xsp + XS_FLUX2]*RDB[xsp + XS_SIGF2];

	      /* Convert fission kappa to correct units */
	      
	      if (((long)RDB[DATA_CODE] == CODE_ARES) ||
		  ((long)RDB[DATA_CODE] == CODE_PARCS))
		{
		  WDB[xsp + XS_EPSF1] = 
		    RDB[xsp + XS_EPSF1]*RDB[xsp + XS_SIGF1]*MEV;
		  WDB[xsp + XS_EPSF2] = 
		    RDB[xsp + XS_EPSF2]*RDB[xsp + XS_SIGF2]*MEV;
		}
	      else if ((long)RDB[DATA_CODE] == CODE_APROS)
		{
		  if ((RDB[xsp + XS_FLUX1] == 0.0) ||
		      (RDB[xsp + XS_FLUX2] == 0.0))
		    Die(NULL, "Zero flux in file %s, universe %s", 
			   GetText(his + HIS_PTR_FILE),
			   GetText(xsp + XS_PTR_GCU));

		  if (fiss1 + fiss2 > 0.0)
		    WDB[xsp + XS_EPSF1] = 
		      MEV*(fiss1*RDB[xsp + XS_EPSF1] +
			   fiss2*RDB[xsp + XS_EPSF2])/(fiss1 + fiss2);

		  /* Flip north and south df's */

		  tmp = RDB[xsp + XS_SDF1N];
		  WDB[xsp + XS_SDF1N] = RDB[xsp + XS_SDF1S];
		  WDB[xsp + XS_SDF1S] = tmp;

		  tmp = RDB[xsp + XS_SDF2N];
		  WDB[xsp + XS_SDF2N] = RDB[xsp + XS_SDF2S];
		  WDB[xsp + XS_SDF2S] = tmp;	
		}
	      
	      /* Include up-scattering in removal */

	      if ((long)RDB[DATA_CODE] != CODE_PARCS)
		{
		  if (RDB[xsp + XS_FLUX1] == 0.0)
		    Die(NULL, "Zero flux in file %s", 
			GetText(his + HIS_PTR_FILE));
		  if (RDB[xsp + XS_FLUX1] > 0.0)
		    WDB[xsp + XS_SIG12] = RDB[xsp + XS_SIG12] -
		      RDB[xsp + XS_SIG21]*RDB[xsp + XS_FLUX2]/
		      RDB[xsp + XS_FLUX1];
		}

	      /* Convert poison cross sections to barn */

	      WDB[xsp + XS_SIGI1] = WDB[xsp + XS_SIGI1]*BARN;
	      WDB[xsp + XS_SIGI2] = WDB[xsp + XS_SIGI2]*BARN;

	      WDB[xsp + XS_SIGX1] = WDB[xsp + XS_SIGX1]*BARN;
	      WDB[xsp + XS_SIGX2] = WDB[xsp + XS_SIGX2]*BARN;

	      WDB[xsp + XS_SIGP1] = WDB[xsp + XS_SIGP1]*BARN;
	      WDB[xsp + XS_SIGP2] = WDB[xsp + XS_SIGP2]*BARN;

	      WDB[xsp + XS_SIGS1] = WDB[xsp + XS_SIGS1]*BARN;
	      WDB[xsp + XS_SIGS2] = WDB[xsp + XS_SIGS2]*BARN;

	      /* Convert yields to one-group */

	      if (fiss1 + fiss2 > 0.0)
		{
		  WDB[xsp + XS_SIGI3] = 
		    (fiss1*RDB[xsp + XS_YLD_I1] + fiss2*RDB[xsp + XS_YLD_I2])/
		    (fiss1 + fiss2);
		  
		  WDB[xsp + XS_SIGX3] = 
		    (fiss1*RDB[xsp + XS_YLD_XE1] + fiss2*RDB[xsp + XS_YLD_XE2])/
		    (fiss1 + fiss2);
		  
		  WDB[xsp + XS_SIGP3] = 
		    (fiss1*RDB[xsp + XS_YLD_PM1] + fiss2*RDB[xsp + XS_YLD_PM2])/
		    (fiss1 + fiss2);
		  
		  WDB[xsp + XS_SIGS3] = 
		    (fiss1*RDB[xsp + XS_YLD_SM1] + fiss2*RDB[xsp + XS_YLD_SM2])/
		    (fiss1 + fiss2);
		  
		  /* Calculate Xe-135 flux */

		  WDB[xsp + XS_XE_FLX] = 2.1184e-05/RDB[xsp + XS_SIGX2];
		  WDB[xsp + XS_SM_FLX] = 3.6271e-06/RDB[xsp + XS_SIGS2];
		}

	      /* Subtract Xe-135 and Sm-149 from absorption */

	      if ((long)RDB[DATA_MODE] != MODE_BAS)
		{
		  WDB[xsp + XS_SIGA1] = WDB[xsp + XS_SIGA1] - 
		    RDB[xsp + XS_MACRO_XE1] - RDB[xsp + XS_MACRO_SM1];
		  
		  WDB[xsp + XS_SIGA2] = WDB[xsp + XS_SIGA2] - 
		    RDB[xsp + XS_MACRO_XE2] - RDB[xsp + XS_MACRO_SM2];
		}

	      /* Set fission spectra if not provided */
	      
	      if (RDB[xsp + XS_CHIP1] + RDB[xsp + XS_CHIP2] == 0.0)
		{
		  WDB[xsp + XS_CHIP1] = 1.0;
		  WDB[xsp + XS_CHIP2] = 0.0;
		}

	      if (RDB[xsp + XS_CHID1] + RDB[xsp + XS_CHID2] == 0.0)
		{
		  WDB[xsp + XS_CHID1] = 1.0;
		  WDB[xsp + XS_CHID2] = 0.0;
		}

	      /* Next */
	      
	      xsp = NextItem(xsp);
	    }
	  
	  /*******************************************************************/

	  /***** Copy nominal values if HZP mode *****************************/

	  /* Check mode */

	  if ((long)RDB[DATA_MODE] == MODE_HZP)
	    {
	      /* Find nominal TFU and TMO */

	      xsp = (long)RDB[his + HIS_PTR_XS];
	      while (xsp > VALID_PTR)
		{
		  /* Compare */

		  if ((RDB[xsp + XS_TFU] == RDB[lib + LIB_NOM_TFU]) &&
		      (RDB[xsp + XS_TMO] == RDB[lib + LIB_NOM_TMO]))
		    {
		      /* Find libraries with matching CR and BOR */

		      ptr = (long)RDB[his + HIS_PTR_XS];
		      while (ptr > VALID_PTR)
			{
			  
			  if (ptr != xsp)
			    if ((RDB[xsp + XS_CR] == RDB[ptr + XS_CR]) &&
				(RDB[xsp + XS_BOR] == RDB[ptr + XS_BOR]))
			      {
				/* Copy data */
			      
				for (i = XS_SIGA1; i < XS_FLUX2 + 1; i++)
				  WDB[ptr + i] = RDB[xsp + i];

			      }
			  
			  /* Next */
			  
			  ptr = NextItem(ptr);
			}
		    }

		  /* Next */
		      
		  xsp = NextItem(xsp);
		}
	      
	    }
	  	  
	  /*******************************************************************/

	  /* Next history */

	  his = NextItem(his);
	}

      /***********************************************************************/

      /***** Set indexes *****************************************************/

      /* Void histories */

      for (i = 0; i < 1000; i++)
	arr[i] = -1;

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Put index */
	      
	      WDB[xsp + XS_VHI_IDX] = ArrayIdx(arr,  RDB[xsp + XS_VHI], 3);

	      /* Next branch */

	      xsp = NextItem(xsp);
	    }

	  /* Next history */

	  his = NextItem(his);
	}

      /* Boron histories */

      for (i = 0; i < 1000; i++)
	arr[i] = -1;

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Put index */
	      
	      WDB[xsp + XS_BHI_IDX] = ArrayIdx(arr,  RDB[xsp + XS_BHI], 3);

	      /* Next branch */

	      xsp = NextItem(xsp);
	    }

	  /* Next history */

	  his = NextItem(his);
	}

      /* Boron branches */

      for (i = 0; i < 1000; i++)
	arr[i] = -1;

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Put index */
	      
	      WDB[xsp + XS_BOR_IDX] = ArrayIdx(arr,  RDB[xsp + XS_BOR], 3);

	      /* Next branch */

	      xsp = NextItem(xsp);
	    }

	  /* Next history */

	  his = NextItem(his);
	}

      /* Control rod branches */

      for (i = 0; i < 1000; i++)
	arr[i] = -1;

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Put index */
	      
	      WDB[xsp + XS_CR_IDX] = ArrayIdx(arr,  RDB[xsp + XS_CR], 3);

	      /* Next branch */

	      xsp = NextItem(xsp);
	    }

	  /* Next history */

	  his = NextItem(his);
	}

      /* Fuel temperature branches */

      for (i = 0; i < 1000; i++)
	arr[i] = -1;

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  xsp = (long)RDB[his + HIS_PTR_XS];
	  
	  while (xsp > VALID_PTR)
	    {
	      /* Put index */
	      
	      WDB[xsp + XS_TFU_IDX] = ArrayIdx(arr,  RDB[xsp + XS_TFU], 3);

	      /* Next branch */

	      xsp = NextItem(xsp);
	    }

	  /* Next history */

	  his = NextItem(his);
	}

      /* Void branches */

      for (i = 0; i < 5; i++)
	arr[i] = -1;

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Put index */

	      WDB[xsp + XS_VOI_IDX] = ArrayIdx(arr,  RDB[xsp + XS_VOI], 5);

	      /* Next branch */

	      xsp = NextItem(xsp);
	    }

	  /* Next history */

	  his = NextItem(his);
	}

      /* Moderator temperature branches */

      for (i = 0; i < 1000; i++)
	arr[i] = -1;

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Put index */
	      
	      WDB[xsp + XS_TMO_IDX] = ArrayIdx(arr,  RDB[xsp + XS_TMO], 3);

	      /* Next branch */

	      xsp = NextItem(xsp);
	    }

	  /* Next history */

	  his = NextItem(his);
	}

      /* Moderator density branches */

      for (i = 0; i < 1000; i++)
	arr[i] = -1;

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Put index */
	      
	      WDB[xsp + XS_RHO_IDX] = ArrayIdx(arr,  RDB[xsp + XS_RHO], 3);

	      /* Next branch */

	      xsp = NextItem(xsp);
	    }

	  /* Next history */

	  his = NextItem(his);
	}

      /* Read number of burnup points from first history */
      
      his = (long)RDB[lib + LIB_PTR_HIS];
      CheckPointer(FUNCTION_NAME, "(his)", DATA_ARRAY, his);

      /* Reset count and previous value */

      nbu = 0;
      val = -1.0;

      xsp = (long)RDB[his + HIS_PTR_XS];
      while (xsp > VALID_PTR)
	{
	  /* Check */

	  if (RDB[xsp + XS_EXP] > val)
	    {
	      /* Update values */

	      val = RDB[xsp + XS_EXP];
	      nbu++;
	    }
	  else
	    break;
	  
	  /* Next branch */
	  
	  xsp = NextItem(xsp);
	}

      /* Put value */

      WDB[lib + LIB_N_EXP] = (double)nbu;

      /* Reset values */

      for (i = 0; i < 1000; i++)
	arr[i] = -1;

      /* Read burnups */

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Put index */
	      
	      WDB[xsp + XS_EXP_IDX] = ArrayIdx(arr,  RDB[xsp + XS_EXP], nbu);

	      /* Next branch */

	      xsp = NextItem(xsp);
	    }

	  /* Next history */

	  his = NextItem(his);
	}

      /***********************************************************************/

      /* Next library */

      lib = NextItem(lib);
    }
  
  /* Newline */

  printf("\n");
}

/*****************************************************************************/

long ArrayIdx(double *arr, double val, long sz)
{
  long n;

  /* Testaa ARES- ja APROS-inputeilla onko toi sz ylipäänsä turha? */
  
  sz = 1000;

  /* Loop over array and find match */

  for (n = 0; n < sz; n++)
    if (val == arr[n])
      break;

  /* Check if found */

  if (n < sz)
    return n;

  /* Put new value in array */

  for (n = 0; n < sz; n++)
    if (arr[n] < 0)
      break;

  /* Check */

  if (n == sz)
    Die(FUNCTION_NAME, "Too many values for array size");
  else
    arr[n] = val;

  /* Return index */

  return n;
}

/*****************************************************************************/
