/*****************************************************************************/
/*                                                                           */
/* AresFit : readf.c                                                         */
/*                                                                           */
/* Created:       2014/12/05 (JLe)                                           */
/* Last modified: 2014/12/05 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: Reads double from file with error checking                   */
/*                                                                           */
/* Comments: - From Serpent 2                                                */
/*                                                                           */
/*****************************************************************************/

#include "header.h"

#define FUNCTION_NAME "ReadF:"

/*****************************************************************************/

double ReadF(FILE *fp)
{
  char str[MAX_STR];
  long n;

  /* Check file pointer */

  if (fp == NULL)
    Die(FUNCTION_NAME, "Null file pointer");

  /* Read next value */

  if (fscanf(fp, "%s", str) == EOF)
    Die(FUNCTION_NAME, "fscanf error");

  /* Loop over string */

  for (n = 0; n < (long)strlen(str); n++)
    {
      /* Check that entry is numerical */

      if (!isdigit(str[n]) && (str[n] != '.') && (str[n] != '-') && 
	  (str[n] != '+') && (str[n] != 'e') && (str[n] != 'E'))
	Die(NULL, "Invalid numerical entry '%s'", str);

      /* Check that + and - are preceded by e */

      if ((n > 0) && ((str[n] == '+') || (str[n] == '-')))
	if ((str[n-1] != 'e') && (str[n-1] != 'E'))
	  Die(NULL, "Invalid numerical entry '%s'", str);
    }

  /* Return number */

  return atof(str);
}

/*****************************************************************************/
