/*****************************************************************************/
/*                                                                           */
/* AresFit : reallocmem.c                                                    */
/*                                                                           */
/* Created:       2014/08/25 (JLe)                                           */
/* Last modified: 2014/08/25 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: Allocates memory                                             */
/*                                                                           */
/* Comments:                                                                 */
/*                                                                           */
/*****************************************************************************/

#include "header.h"
#include "locations.h"

#define FUNCTION_NAME "ReallocMem:"

/*****************************************************************************/

long ReallocMem(long type, long sz)
{
  long memsize, realsize, block, loc0, totsize;

  /* Check size */

  if (sz < 1)
    Die (FUNCTION_NAME, "Invalid data size %ld (type %ld)", sz, type);
    
  /* Avoid compiler warning */

  realsize = 0;
  memsize = 0;

  /* Allocate memory in 20MB blocks */
  
  block = (long)(20.0*MEGA/sizeof(double));

  /***************************************************************************/
  
  /***** Get size of datablock ***********************************************/
  
  if (type == DATA_ARRAY)
    {
      if (WDB != NULL)
	{
	  memsize = (long)RDB[DATA_ALLOC_MAIN_SIZE];
	  realsize = (long)RDB[DATA_REAL_MAIN_SIZE];
	}
      else
	{
	  memsize = 0;
	  realsize = 0;
	}
    }
  else
    Die(FUNCTION_NAME, "Invalid data type %ld", type);

  /* Remember pointer */

  loc0 = memsize;

  /* Increase size */
  
  memsize = memsize + sz;

  /***************************************************************************/
  
  /***** Check limit *********************************************************/

  /* Check pointer */

  if (WDB != NULL)
    { 
      /* Get total size */
      
      totsize = (long)RDB[DATA_TOTAL_BYTES];

      if (memsize > realsize)
	totsize = totsize + block*sizeof(double);
    }

  /***************************************************************************/
  
  /***** Allocate memory for main data array *********************************/

  if (type == DATA_ARRAY)
    {
      /* Compare to real data size */

      if (memsize > realsize)
	{
	  /* Increase real memory size */

	  realsize = memsize + block;

	  /* Allocate more memory */
	  
	  WDB = (double *)Mem(MEM_REALLOC, WDB, realsize*sizeof(double));
	  
	  /* Initialize values */
	  
	  memset(&WDB[loc0], 0.0, (sz + block)*sizeof(double));

	  /* Update real data size */

	  WDB[DATA_REAL_MAIN_SIZE] = (double)realsize;
	}

      /* Update block size */
      
      WDB[DATA_ALLOC_MAIN_SIZE] = (double)memsize;
    }

  /***************************************************************************/
  
  /* Put read-only pointers */

  RDB = (const double *)WDB;

  /* Return pointer to allocated memory */

  return loc0;
}

/*****************************************************************************/
