/*****************************************************************************/
/*                                                                           */
/* AresFit : reconfigure.c                                                   */
/*                                                                           */
/* Created:       2015/02/23 (JLe)                                           */
/* Last modified: 2015/02/23 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: Reconfigures libraries from runs with multiple gcu's         */
/*                                                                           */
/* Comments:                                                                 */
/*                                                                           */
/*****************************************************************************/

#include "header.h"
#include "locations.h"

#define FUNCTION_NAME "Reconfigure:"

/*****************************************************************************/

void Reconfigure()
{
  long lib, his, new, xsp, ptr, n;

  /* Reset temporary pointer */

  WDB[DATA_TMP_PTR] = -1.0;

  /* Loop over data */

  xsp = (long)RDB[DATA_PTR_XS0];
  while (xsp > VALID_PTR)
    {
      /* Pointer to original library */

      ptr = (long)RDB[xsp + XS_PTR_LIB];
      CheckPointer(FUNCTION_NAME, "(ptr)", DATA_ARRAY, ptr);

      /* Find match */

      lib = (long)RDB[DATA_TMP_PTR];
      while (lib > VALID_PTR)
	{
	  /* Compare gcu and library names */
	  
	  if (CompareStr(lib + LIB_PTR_GCU, xsp + XS_PTR_GCU))
	    if (CompareStr(lib + LIB_PTR_NAME, ptr + LIB_PTR_NAME))
	      break;

	  /* Next */

	  lib = NextItem(lib);
	}

      /* Check if found */
      
      if (lib < VALID_PTR)
	{
	  /* Create new */
		  
	  lib = NewItem(DATA_TMP_PTR, LIB_BLOCK_SIZE);

	  /* Copy data */

	  for (n = LIST_DATA_SIZE; n < LIB_BLOCK_SIZE; n++)
	    WDB[lib + n] = RDB[ptr + n];

	  /* Put universe name and reset pointer to histories */

	  WDB[lib + LIB_PTR_GCU] = RDB[xsp + XS_PTR_GCU];
	  WDB[lib + LIB_PTR_HIS] = 0.0;
	}

      /* Reconfigure pointer */

      WDB[xsp + XS_PTR_LIB] = (double)lib;

      /* Pointer to original history */

      ptr = (long)RDB[xsp + XS_PTR_HIS];
      CheckPointer(FUNCTION_NAME, "(ptr)", DATA_ARRAY, ptr);

      /* Find matching history */

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  /* Compare file name (unique to history) */
	  
	  if (CompareStr(his + HIS_PTR_FILE, ptr + HIS_PTR_FILE))
	      break;

	  /* Next */

	  his = NextItem(his);
	}
      
      /* Check if found */
      
      if (his < VALID_PTR)
	{
	  /* Create new */
		  
	  his = NewItem(lib + LIB_PTR_HIS, HIS_BLOCK_SIZE);

	  /* Copy data */

	  for (n = LIST_DATA_SIZE; n < HIS_BLOCK_SIZE; n++)
	    WDB[his + n] = RDB[ptr + n];

	  /* Put universe name and reset pointer to data */

	  WDB[his + HIS_PTR_GCU] = RDB[xsp + XS_PTR_GCU];
	  WDB[his + HIS_PTR_XS] = 0.0;
	}
      
      /* Reconfigure pointer */
      
      WDB[xsp + XS_PTR_HIS] = (double)his;

      /* Create new item */

      new = NewItem(his + HIS_PTR_XS, XS_BLOCK_SIZE);

      /* Copy data */

      for (n = LIST_DATA_SIZE; n < XS_BLOCK_SIZE; n++)
	WDB[new + n] = RDB[xsp + n];

      /* Next */

      xsp = NextItem(xsp);
    }

  /* Change pointer */

  WDB[DATA_PTR_LIB0] = RDB[DATA_TMP_PTR];

  /* Final checks */

  lib = (long)RDB[DATA_PTR_LIB0];
  while (lib > VALID_PTR)
    {
      /* Loop over histories */

      his = (long)RDB[lib + LIB_PTR_HIS];
      while (his > VALID_PTR)
	{
	  /* Check gcu name */

	  if (!CompareStr(lib + LIB_PTR_GCU, his + HIS_PTR_GCU))
	    Die(FUNCTION_NAME, "Mismatch in gcu name");

	  /* Loop over branches */

	  xsp = (long)RDB[his + HIS_PTR_XS];
	  while (xsp > VALID_PTR)
	    {
	      /* Check gcu name */
		     
	      if (!CompareStr(his + HIS_PTR_GCU, xsp + XS_PTR_GCU))
		Die(FUNCTION_NAME, "Mismatch in gcu name");

	      /* Next branch */

	      xsp = NextItem(xsp);
	    }

	  /* Next history */

	  his = NextItem(his);
	}

      /* Next library */

      lib = NextItem(lib);
    }

  /***************************************************************************/
}

/*****************************************************************************/
