/*****************************************************************************/
/*                                                                           */
/* AresFit : sortlist.c                                                      */
/*                                                                           */
/* Created:       2015/02/23 (JLe)                                           */
/* Last modified: 2015/02/23 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: - Sorts list items                                           */
/*                                                                           */
/* Comments: - From Serpent 2                                                */
/*                                                                           */
/*****************************************************************************/

#include "header.h"
#include "locations.h"

#define FUNCTION_NAME "SortList:"

/*****************************************************************************/

void SortList(long lst, long param, long mode)
{
  long ptr, loc0, next, count;
  double val1, val2;

  /* Check pointer */

  CheckPointer(FUNCTION_NAME, "(lst)", DATA_ARRAY, lst);
  
  /* Go to first item */
  
  lst = FirstItem(lst);

  /* Sort loop */

  do
    {
      /* Reset count */

      count = 0;

      /* Check mode */

      if (mode == SORT_MODE_ASCEND)
	{
	  /***** Sort items in ascending order *******************************/

	  /* Set starting point for iteration */

	  loc0 = FirstItem(lst);

	  /* Sorting loop */
	  
	  while (loc0 > VALID_PTR)
	    {
	      /* Set pointers for current and next */

	      ptr = loc0;
	      next = NextItem(ptr);

	      /* Set starting point for next iteration */
	      
	      loc0 = next;

	      /* Loop over list */
	  
	      while (next > VALID_PTR)
		{
		  /* Get values */

		  val1 = RDB[ptr + param];
		  val2 = RDB[next + param];
		  
		  /* Compare items, move one step forward or break loop */
		  
		  if (val1 > val2)
		    {
		      MoveItemRight(ptr);
		      count++;
		    }
		  else
		    break;
		  
		  /* Next */
		  
		  next = NextItem(ptr);
		}
	    }

	  /*******************************************************************/
	}
      else if(mode == SORT_MODE_DESCEND)
	{
	  /***** Sort items in descending order ******************************/

	  /* Set starting point for iteration */

	  loc0 = FirstItem(lst);

	  /* Sorting loop */
	  
	  while (loc0 > VALID_PTR)
	    {
	      /* Set pointers for current and next */

	      ptr = loc0;
	      next = NextItem(ptr);

	      /* Set starting point for next iteration */
	      
	      loc0 = next;

	      /* Loop over list */
	  
	      while (next > VALID_PTR)
		{
		  /* Get values */

		  val1 = RDB[ptr + param];
		  val2 = RDB[next + param];

		  /* Compare items, move one step forward or break loop */
		  
		  if (val1 < val2)
		    {
		      MoveItemRight(ptr);
		      count++;
		    }
		  else
		    break;
		  
		  /* Next */
		  
		  next = NextItem(ptr);
		}
	    }

	  /*******************************************************************/
	}
      else
	Die(FUNCTION_NAME, "Invalid sort mode");
    }
  while (count > 0);

#ifdef DEBUG

  /* Check order */

  ptr = FirstItem(lst);

  while (ptr > 0)
    {
      if ((next = NextItem(ptr)) > 0)
	{
	  /* Get values */
	      
	  val1 = RDB[ptr + param];
	  val2 = RDB[next + param];
	  
	  if ((mode == SORT_MODE_ASCEND) && (val1 > val2))
	    Die(FUNCTION_NAME, "Sort error");
	  else if ((mode == SORT_MODE_DESCEND) && (val1 < val2))
	    Die(FUNCTION_NAME, "Sort error");
	}
      else
	break;
      
      /* Next */

      ptr = next;
    }

#endif

  /* Set direct pointers if list is closed */

  if ((long)RDB[lst + LIST_PTR_DIRECT] > VALID_PTR)
    SetDirectPointers(lst);
}

/*****************************************************************************/
