/*****************************************************************************/
/*                                                                           */
/* AresFit : testapros.c                                                     */
/*                                                                           */
/* Created:       2014/12/17 (JLe)                                           */
/* Last modified: 2014/12/17 (JLe)                                           */
/* Version:       1.0.0                                                      */
/*                                                                           */
/* Description: Tests Apros polynomial fit                                   */ 
/*                                                                           */
/* Comments:                                                                 */
/*                                                                           */
/*****************************************************************************/

#include "header.h"
#include "locations.h"

#define FUNCTION_NAME "TestApros:"

/*****************************************************************************/

void TestApros(long lib, long para, double VHI, double CR, double EXP, 
	       double BHI, double *dat, long ntf, long ntn, long nbo)
{
  long his, ptr, n;
  double x1, x2, x3, val;
  double BOR, TFU, TMO, coe[7];

  /* reset polynomial coefficients */

  for (n = 0; n < 7; n++)
    coe[n] = 0.0;
  
  /* Put values */
  
  n = 0;
  coe[0] = dat[n++];
  
  if (ntf == 3)
    {
      coe[1] = dat[n++];
      coe[2] = dat[n++];
    }
  if (ntn == 3)
    {
      coe[3] = dat[n++];
      coe[4] = dat[n++];
    }
  if (nbo == 3)
    {
      coe[5] = dat[n++];
      coe[6] = dat[n++];
    }

  
  /* Loop over history */

  his = (long)RDB[lib + LIB_PTR_HIS];
  while (his > VALID_PTR)
    {
      /* Loop over branches */
      
      ptr = (long)RDB[his + HIS_PTR_XS];
      CheckPointer(FUNCTION_NAME, "(ptr)", DATA_ARRAY, ptr);
      
      /* Check VHI, CR, EXP and BHI */
      
      if ((VHI != RDB[ptr + XS_VHI]) ||
	  (CR != RDB[ptr + XS_CR]) ||
	  (EXP != RDB[ptr + XS_EXP]) ||
	  (BHI != RDB[ptr + XS_BHI]))
	{
	  /* Pointer to next */
	  
	  his = NextItem(his);
	  
	  /* Cycle loop */
	  
	  continue;
	}
      
      /* Loop over branches */

      while (ptr > VALID_PTR)
	{
	  /* Get remaining state variables */

	  /* HUOM: tiheys !!! */

	  BOR = RDB[ptr + XS_BOR];
	  TFU = RDB[ptr + XS_TFU];
	  TMO = RDB[ptr + XS_RHO];
	  
	  /* Get value */
      
	  val = GetXS(lib, para, VHI, BHI, BOR, CR, TFU, 0.0, TMO, EXP);

	  /* Compare value */
	  
	  if (val != RDB[ptr + para])
	    Die(FUNCTION_NAME, "Mismatch in value");

	  if (TMO != RDB[lib + LIB_NOMCON_TMNOM])
	    if ((para == XS_DIFF1) || (para == XS_DIFF2))
	      val = 1.0/val;

	  /* Get variables */

      	  /* HUOM: tiheys !!! */

	  x1 = sqrt(TFU) - sqrt(RDB[lib + LIB_NOMCON_TFNOM]);
	  x2 = TMO - RDB[lib + LIB_NOMCON_DMNOM];
	  x3 = BOR - RDB[lib + LIB_NOMCON_CBNOM];

	  /* Check difference */
	  
	  if (val > 0.0)
	    if (fabs(AprosPoly(coe, x1, x2, x3)/val - 1) > 1E-9)
	      {
		/* Print state */
		
		printf("TFU = %E TMO = %E BOR = %E  ", x1, x2, x3);
		
		/* Print parameter name */

		if (para == XS_SIGA1)
		  printf("   SIGA1 :");
		else if (para == XS_SIGA2)
		  printf("   SIGA2 :");
		else if (para == XS_NUSF1)
		  printf("   NUSF1 :");
		else if (para == XS_NUSF2)
		  printf("   NUSF2 :");
		else if (para == XS_DIFF1)
		  printf("   DIFF1 :");
		else if (para == XS_DIFF2)
		  printf("   DIFF2 :");
		else if (para == XS_SIG12)
		  printf("   SIG12 :");
		else if (para == XS_EPSF1)
		  printf("   EPSF1 :");
		else if (para == XS_EPSF2)
		  printf("   EPSF2 :");
		else if (para == XS_SIGF1)
		  printf("   SIGF1 :");
		else if (para == XS_SIGF2)
		  printf("   SIGF2 :");
		else if (para == XS_SDF1W)
		  printf("   SDF1W :");
		else if (para == XS_SDF2W)
		  printf("   SDF2W :");
		else if (para == XS_SDF1S)
		  printf("   SDF1S :");
		else if (para == XS_SDF2S)
		  printf("   SDF2S :");
		else if (para == XS_SDF1E)
		  printf("   SDF1E :");
		else if (para == XS_SDF2E)
		  printf("   SDF2E :");
		else if (para == XS_SDF1N)
		  printf("   SDF1N :");
		else if (para == XS_SDF2N)
		  printf("   SDF2N :");
		else if (para == XS_CD2SE)
		  printf("   CD2SE :");
		else if (para == XS_CD1NE)
		  printf("   CD1NE :");
		else if (para == XS_CD2NE)
		  printf("   CD2NE :");
		else if (para == XS_CD1NW)
		  printf("   CD1NW :");
		else if (para == XS_CD2NW)
		  printf("   CD2NW :");
		else if (para == XS_CD1SW)
		  printf("   CD1SW :");
		else if (para == XS_CD2SW)
		  printf("   CD2SW :");
		else if (para == XS_CD1SE)
		  printf("   CD1SE :");
		else if (para == XS_CD2SE)
		  printf("   CD2SE :");
		else if (para == XS_SIGI1)
		  printf("   SIGI1 :");
		else if (para == XS_SIGI2)
		  printf("   SIGI2 :");
		else if (para == XS_SIGI3)
		  printf("   SIGI3 :");
		else if (para == XS_SIGX1)
		  printf("   SIGX1 :");
		else if (para == XS_SIGX2)
		  printf("   SIGX2 :");
		else if (para == XS_SIGX3)
		  printf("   SIGX3 :");
		else if (para == XS_SIGP1)
		  printf("   SIGP1 :");
		else if (para == XS_SIGP2)
		  printf("   SIGP2 :");
		else if (para == XS_SIGP3)
		  printf("   SIGP3 :");
		else if (para == XS_SIGS1)
		  printf("   SIGS1 :");
		else if (para == XS_SIGS2)
		  printf("   SIGS2 :");
		else if (para == XS_SIGS3)
		  printf("   SIGS3 :");
		else if (para == XS_BETA )
		  printf("   BETA  :");
		else if (para == XS_BETA1)
		  printf("   BETA1 :");
		else if (para == XS_BETA2)
		  printf("   BETA2 :");
		else if (para == XS_BETA3)
		  printf("   BETA3 :");
		else if (para == XS_BETA4)
		  printf("   BETA4 :");
		else if (para == XS_BETA5)
		  printf("   BETA5 :");
		else if (para == XS_BETA6)
		  printf("   BETA6 :");
		else if (para == XS_INVV1)
		  printf("   INVV1 :");
		else if (para == XS_INVV2)
		  printf("   INVV2 :");
		else if (para == XS_LBDA1)
		  printf("   LBDA1 :");
		else if (para == XS_LBDA2)
		  printf("   LBDA2 :");
		else if (para == XS_LBDA3)
		  printf("   LBDA3 :");
		else if (para == XS_LBDA4)
		  printf("   LBDA4 :");
		else if (para == XS_LBDA5)
		  printf("   LBDA5 :");
		else if (para == XS_LBDA6)
		  printf("   LBDA6 :");
		else
		  Die(FUNCTION_NAME, "Overflow %ld", para - LIST_DATA_SIZE);
	    
		/* Print difference */
		
		printf("%E %E : %7.2f\n", val, AprosPoly(coe, x1, x2, x3), 
		       100*(AprosPoly(coe, x1, x2, x3)/val - 1));
	      }

	  /* Next branch */
	  
	  ptr = NextItem(ptr);
	}

      /* Next history */

      his = NextItem(his);
    }
}

/*****************************************************************************/
